""" An extension to the original PyUnit providing specific Device Pool test
    utilities """

import logging
import unittest
import PyTango
import sys
import os
import subprocess
import time
import signal
import imp

try:
    import pexpect
except:
    print("The Pool Unit test requires pexpect python module "
          "which was not found.")
    print("This module can be found at http://www.noah.org/wiki/Pexpect")
    sys.exit(-1)


class DefaultTangoEventCallBack:

    def __init__(self):
        self.evt_list = []
        self.cb_executed = 0
        self.cb_error = 0
        self.verbose = False

    def push_event(self, event):
        self.cb_executed += 1
        self.evt_list.append(event)
        if event.err:
            self.cb_error += 1
        elif self.verbose:
            sys.stderr.write("%s\n" % str(event.attr_value.value))
            sys.stderr.flush()

    def getEventValues(self):
        ret = []
        for e in self.evt_list:
            if e.err:
                ret.append(None)
            else:
                ret.append(e.attr_value.value)
        return ret


class PoolTestCase(unittest.TestCase):
    """An extension to the original PyUnit TestCase providing specific methods
    for DevicePool tests"""

    ReadyMsg = "Ready to accept request"
    AlreadyRunning = "This server is already running, exiting!"
    PoolExecNotFound = "No such file or directory"
    PoolReadyMsg = ReadyMsg
    PoolAlreadyRunning = AlreadyRunning
    PoolMaxStartupTime = 10
    PoolMaxShutdownTime = 10

    MotorSimulatorReadyMsg = ReadyMsg
    MotorSimulatorAlreadyRunning = AlreadyRunning
    MotorSimulatorMaxStartupTime = 10
    MotorSimulatorMaxShutdownTime = 10

    CounterTimerSimulatorReadyMsg = ReadyMsg
    CounterTimerSimulatorAlreadyRunning = AlreadyRunning
    CounterTimerSimulatorMaxStartupTime = 10
    CounterTimerSimulatorMaxShutdownTime = 10

    def check_empty_attribute(self, dev, att_name):
        """ check_empty_attribute(PyTango.DeviceProxy dev, string att_name) -> None

            Reads the given attribute from the given device and checks that the
            received value is empty.

            Paramaters:
                dev: a PyTango.DeviceProxy
                att_name: a string containing the attribute name
        """
        try:
            c_list = dev.read_attribute(att_name)
            if len(c_list.value) != 0:
                self.assert_(False, "The %s attribute is not empty !! It contains: %s" % (
                    att_name, c_list.value))
        except PyTango.DevFailed as e:
            except_value = sys.exc_info()[1]
            self.assertEqual(
                except_value[0]["reason"], "API_EmptyDeviceAttribute")
            self.assertEqual(
                except_value[0]["desc"], "cannot extract, no data in DeviceAttribute object ")

    def attribute_error(self, dev, att_name, err, pr=False):
        """ attribute_error(PyTango.DeviceProxy dev, string att_name, string err, bool pr = False)

            Reads the given attribute from the given device and checks that this
            operation generates a PyTango.DevFailed exception with its 'reason'
            being equal to the given err string. If pr is set to True then the
            exception description is sent to the output.
        """
        try:
            c_list = dev.read_attribute(att_name)
            self.assert_(
                False, "The %s attribute is not in fault!!" % (att_name))
        except PyTango.DevFailed as e:
            except_value = sys.exc_info()[1]
            if pr:
                self._printException(except_value)
            self.assertEqual(except_value[0]["reason"], err)

    def wr_attribute_error(self, dev, att_val, err, pr=False):
        """ wr_attribute_error(PyTango.DeviceProxy dev, PyTango.AttributeValue att_value, string err, bool pr = False)

            Writes the given attribute from the given device with the given
            value and checks that this operation generates a PyTango.DevFailed
            exception with its 'reason' being equal to the given err string.
            If pr is set to True then the exception description is sent to the
            output.
        """
        try:
            dev.write_attribute(att_val)
            self.assert_(False, "The %s attribute is not in fault!!" %
                         (att_val.name))
        except PyTango.DevFailed as e:
            except_value = sys.exc_info()[1]
            if pr:
                self._printException(except_value)
            self.assertEqual(except_value[0]["reason"], err)

    def wrong_argument(self, dev, cmd_name, arg_list, err, pr=False):
        """ wrong_argument(PyTango.DeviceProxy dev, string cmd_name, list arg_list, string err, bool pr = False)

            Executes the given command with the given arguments on the given
            device and expects a PyTango.DevFailed exception to be raised with
            its 'reason' being equal to the given err string.
            If pr is set to True then the exception description is sent to the
            output.
        """
        try:
            dev.command_inout(cmd_name, arg_list)
            self.assert_(
                False, "The %s command succeed "
                       "with wrong arguments!!" % cmd_name)
        except PyTango.DevFailed as e:
            except_value = sys.exc_info()[1]
            if pr:
                self._printException(except_value)
            self.assertEqual(except_value[0]["reason"], err)

    def _write_attribute(self, dev, att_name, att_val):
        """ _write_attribute(PyTango.DeviceProxy dev, string att_name, object attr_val)

            Writes the given attribute from the given device with the given
            value.
        """
        val = PyTango.AttributeValue()
        val.name = att_name
        val.value = att_val
        dev.write_attribute(val)

    def _printException(self, except_value):
        print("\nERROR desc")
        print("origin =", except_value[0]["origin"])
        print("desc =", except_value[0]["desc"])
        print("origin =", except_value[0]['origin'])

    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~
    # Default setup. Overwrite this methods in each
    # test scenario when necessary
    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~
    def getPoolPath(self):
        """getPoolPath() -> list<string>
        """
        lib_root = os.path.abspath(
            os.path.join(self.pool_bin_dir, "..", "lib"))

        pydir = os.path.join(lib_root, "python")
        if not os.path.exists(pydir):
            pydir += "%d.%d" % (sys.version_info[0], sys.version_info[1])

        test_ctrl_dir = os.path.abspath(os.path.curdir)
        test_ctrl_dir = os.path.join(test_ctrl_dir, "test_ctrl")

        return [os.path.join(lib_root, "pool"),
                test_ctrl_dir,
                os.path.join(lib_root, "poolcontrollers"),
                os.path.join(pydir, "site-packages")]

    def getMotorControllerSimulatorMaxError(self):
        return 0

    def getMotorSimulators(self):
        return 10 * ({"properties": {"_Velocity": ['5.0'],
                                     "_Acceleration": ['2.0'],
                                     "_Deceleration": ['2.0'],
                                     "_Base_rate": ['0.01']},
                      },)

    def getCounterTimerSimulators(self):
        ret = []
        for i in range(10):
            mot_name = "%s/simmot/test%03d" % (self.username, i + 1)
            ret.append({"properties": {"Average": ['1.0'],
                                       "Sigma": ['250.0'],
                                       "MotorName": [mot_name]},
                        },)
        return ret

    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Generic methods
    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def deleteFromDB(self, server_name, server_instance):
        server_name = server_name.lower()
        server_instance = server_instance.lower()
        server = server_name + '/' + server_instance
        servers = [s.lower()
                   for s in self.tango_db.get_server_list(server_name + '/*')]
        if server in servers:
            devices = self.tango_db.get_device_name(server, '*')

            for d in devices:
                if not d.startswith('dserver'):
                    props = self.tango_db.get_device_property_list(d, '*')
                    self.tango_db.delete_device_property(d, props)
                    self.tango_db.delete_device(d)

            self.tango_db.delete_server(server)

    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Test requirements. Overwrite as necessary
    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def needsPool(self):
        return True

    def needsMotorSimulator(self):
        return False

    def needsCounterTimerSimulator(self):
        return False

    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Pre and Post test methods
    # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def setUp(self):
        """Default setUp."""
        self.prepare()
        self.start()

    def tearDown(self):
        """Default tearDown."""
        self.stop()
        self.cleanUp()

    def start(self):
        """Default startup. Starts Simulators and Device Pool"""
        self.startMotorSimulator()
        self.startCounterTimerSimulator()
        self.startPool()

    def stop(self):
        """Default stop. Stops Simulators and Device Pool"""
        self.stopPool()
        self.stopCounterTimerSimulator()
        self.stopMotorSimulator()

    def prepare(self):
        """Prepares everything to be run"""
        self.tango_db = PyTango.Database()
        self.username = os.getlogin()
        self.common_ds_instance = self.username + "_test"
        self.prepareMotorSimulator()
        self.prepareCounterTimerSimulator()
        self.prepareDevicePool()

    def cleanUp(self):
        """Cleans the Database"""
        self.deletePoolFromDB()
        self.deleteCounterTimerSimulatorFromDB()
        self.deleteMotorSimulatorFromDB()

    def prepareMotorSimulator(self):
        """Prepare the Motor Simulator local variables and registers in the Database"""

        if not self.needsMotorSimulator():
            return

        self.motsim_ds_instance = self.common_ds_instance
        self.motsim_ctrl_dev_name = self.username + "/simmotctrl/test"
        self.motsim_process = None
        self.motsim_start_time = -1
        self.motsim_stopt_time = -1
        self.motsim_exec = None

        # Make sure the python device server code is reachable
        try:
            f, self.motsim_exec, desc = imp.find_module('SimuMotorCtrl')
            f.close()
            f, path, desc = imp.find_module('SimuMotor')
            f.close()
        except ImportError as e:
            self.assertTrue(False, e)
            logging.log_exception(e)
        # Include the name and path attributes in output.
        logging.log('error.name: {e.name}')
        logging.log('error.path: {e.path}')

        # Cleanup the database
        self.deleteMotorSimulatorFromDB()

        # Create a new instance in database
        self.createMotorSimulatorInDB()

    def createMotorSimulatorInDB(self):
        """Registers the Motor Simulator in the Database"""

        if not self.needsMotorSimulator():
            return

        # Create the motor controller simulator
        ctrl_server_info = PyTango.DbDevInfo()
        ctrl_server_info.name = self.motsim_ctrl_dev_name
        ctrl_server_info._class = 'SimuMotorCtrl'
        ctrl_server_info.server = 'SimuMotorCtrl/' + self.motsim_ds_instance
        self.tango_db.add_device(ctrl_server_info)

        # Add the MaxError property
        props = {'MaxError': [str(self.getMotorControllerSimulatorMaxError())]}
        self.tango_db.put_device_property(ctrl_server_info.name, props)

        # Create the motor simulators
        mots = self.getMotorSimulators()

        mot_idx = 1
        for mot in mots:
            mot_info = PyTango.DbDevInfo()
            mot_info.name = "%s/simmot/test%03d" % (self.username, mot_idx)
            mot_info._class = 'SimuMotor'
            mot_info.server = 'SimuMotorCtrl/' + self.motsim_ds_instance
            self.tango_db.add_device(mot_info)
            self.tango_db.put_device_property(mot_info.name, mot['properties'])
            mot_idx += 1

    def deleteMotorSimulatorFromDB(self):
        """Unregisters the Motor Simulator from the Database"""

        if not self.needsMotorSimulator():
            return

        self.deleteFromDB('SimuMotorCtrl', self.motsim_ds_instance)

    def prepareCounterTimerSimulator(self):
        """Prepare the Counter Timer Simulator local variables and registers in the Database"""

        if not self.needsCounterTimerSimulator():
            return

        self.ctsim_ds_instance = self.common_ds_instance
        self.ctsim_ctrl_dev_name = self.username + "/simctctrl/test"
        self.ctsim_process = None
        self.ctsim_start_time = -1
        self.ctsim_stopt_time = -1
        self.ctsim_exec = None

        # Make sure the python device server code is reachable
        try:
            f, self.ctsim_exec, desc = imp.find_module('SimuCoTiCtrl')
            f.close()
            f, path, desc = imp.find_module('SimuCounter')
            f.close()
        except ImportError as e:
            self.assert_(False, e)
            logging.log_exception(e)
        # Include the name and path attributes in output.
        logging.log('error.name: {e.name}')
        logging.log('error.path: {e.path}')

        self.ctsim_exec = self.ctsim_exec

        # Cleanup the database
        self.deleteCounterTimerSimulatorFromDB()

        # Create a new instance in database
        self.createCounterTimerSimulatorInDB()

    def createCounterTimerSimulatorInDB(self):
        """Registers the Counter Timer Simulator in the Database"""

        if not self.needsCounterTimerSimulator():
            return

        # Create the motor controller simulator
        ctrl_server_info = PyTango.DbDevInfo()
        ctrl_server_info.name = self.ctsim_ctrl_dev_name
        ctrl_server_info._class = 'SimuCoTiCtrl'
        ctrl_server_info.server = 'SimuCoTiCtrl/' + self.ctsim_ds_instance
        self.tango_db.add_device(ctrl_server_info)

        # Create the counter timer simulators
        cts = self.getCounterTimerSimulators()

        ct_idx = 1
        for ct in cts:
            ct_info = PyTango.DbDevInfo()
            ct_info.name = "%s/simct/test%03d" % (self.username, ct_idx)
            ct_info._class = 'SimuCounter'
            ct_info.server = 'SimuCoTiCtrl/' + self.ctsim_ds_instance
            self.tango_db.add_device(ct_info)
            self.tango_db.put_device_property(ct_info.name, ct['properties'])
            ct_idx += 1

    def deleteCounterTimerSimulatorFromDB(self):
        """Unregisters the Counter Timer Simulator from the Database"""

        if not self.needsCounterTimerSimulator():
            return

        self.deleteFromDB('SimuCoTiCtrl', self.ctsim_ds_instance)

    def prepareDevicePool(self):
        """Prepare the Device Pool local variables and registers in the Database"""

        if not self.needsPool():
            return

        self.pool_ds_instance = self.common_ds_instance
        self.pool_dev_name = self.username + "/pool/test"
        self.pool_process = None
        self.pool_start_time = -1
        self.pool_stopt_time = -1
        self.pool_exec = None

        for path in os.getenv("PATH").split(":"):
            path = os.path.join(path, "Pool")
            if os.path.exists(path):
                self.pool_exec = path
                break

        self.assertFalse(self.pool_exec is None,
                         "Could not find Pool executable. "
                         "Make sure it is in the PATH")

        self.pool_bin_dir = os.path.dirname(self.pool_exec)

        self.deletePoolFromDB()

        self.createPoolInDB()

        self.pool_dp = PyTango.DeviceProxy(self.pool_dev_name)

    def createPoolInDB(self):
        """Registers the Device Pool in the Database"""

        if not self.needsPool():
            return

        # Create the device pool in the database
        pool_server_info = PyTango.DbDevInfo()
        pool_server_info.name = self.pool_dev_name
        pool_server_info._class = 'Pool'
        pool_server_info.server = 'Pool/' + self.pool_ds_instance
        self.tango_db.add_device(pool_server_info)

        # Add the PoolPath property
        props = {'PoolPath': self.getPoolPath()}
        self.tango_db.put_device_property(pool_server_info.name, props)

    def deletePoolFromDB(self):
        """Unregisters the Device Pool from the Database"""

        if not self.needsPool():
            return

        self.pool_dp = None

        self.deleteFromDB('Pool', self.pool_ds_instance)

    def waitPoolStop(self):
        """Waits for the Device Pool process (if any) to stop orderly."""
        if self.pool_process is None:
            return 0

        idx = self.pool_process.expect(["Exiting",
                                        pexpect.EOF,
                                        pexpect.TIMEOUT],
                                       timeout=PoolTestCase.PoolMaxShutdownTime)
        if idx == 0:
            pass
        elif idx == 1:
            self.assert_(
                False, "Device Pool terminated unexpectedly (before 'Exiting')")
        elif idx == 2:
            self.assert_(False, "Device Pool shutdown time exceeded")

        idx = self.pool_process.expect(["Exited",
                                        pexpect.EOF,
                                        pexpect.TIMEOUT],
                                       timeout=PoolTestCase.PoolMaxShutdownTime)
        if idx == 0:
            pass
        elif idx == 1:
            self.assert_(
                False, "Device Pool terminated unexpectedly (before 'Exited')")
        elif idx == 2:
            self.assert_(False, "Device Pool shutdown time exceeded")

        try:
            ret = self.pool_process.wait()
            self.pool_process = None
        except:
            # if the process has already terminated before the call to wait()
            ret = self.pool_process.exitstatus
            self.pool_process = None

        self.failUnless(
            ret == 0, "Device Pool terminated with exitcode = %d" % ret)

        return ret

    def startPool(self):
        """Starts the Device pool"""

        if not self.needsPool():
            return

        self.pool_process = pexpect.spawn(self.pool_exec,
                                          args=[self.pool_ds_instance])
        # make sure the device pool sends events
        self.pool_process.delayafterterminate = 2

        idx = self.pool_process.expect([PoolTestCase.PoolReadyMsg,
                                        PoolTestCase.PoolAlreadyRunning,
                                        pexpect.EOF,
                                        pexpect.TIMEOUT],
                                       timeout=PoolTestCase.PoolMaxStartupTime)

        if idx == 0:
            return
        elif idx == 1:
            self.assertTrue(False, PoolTestCase.PoolAlreadyRunning)
        elif idx == 2:
            self.assertTrue(False, "Device Pool terminated unexpectedly")
        elif idx == 3:
            self.assertTrue(False, "Device Pool startup time exceeded")

    def stopPool(self):
        """Stops the Device Pool"""

        if not self.needsPool():
            return

        if self.pool_process is None:
            return

        if not self.pool_process.isalive():
            return

        self.pool_process.terminate()
        if self.pool_process.isalive():
            # flush any existing output
            self.pool_process.close()
            if not self.pool_process.isalive():
                ret = self.pool_process.exitstatus
                self.failUnless(
                    ret == 0, "Device Pool terminated with exitcode = %d" % ret)
                return
        else:
            ret = self.pool_process.exitstatus
            self.failUnless(
                ret == 0, "Device Pool terminated with exitcode = %d" % ret)
            return

        self.pool_process.terminate(force=True)
        if not self.pool_process.isalive():
            self.assert_(False, "Device Pool hangs: Forced to do a SIGKILL")

        self.assert_(False, "Device Pool hangs EVEN with SIGKILL")

    def startMotorSimulator(self):
        """Starts the Motor Simulator"""

        if not self.needsMotorSimulator():
            return

        self.motsim_process = pexpect.spawn("python",
                                            args=[self.motsim_exec,
                                                  self.motsim_ds_instance])

        idx = self.motsim_process.expect([PoolTestCase.MotorSimulatorReadyMsg,
                                          PoolTestCase.MotorSimulatorAlreadyRunning,
                                          pexpect.EOF,
                                          pexpect.TIMEOUT],
                                         timeout=PoolTestCase.MotorSimulatorMaxStartupTime)

        if idx == 0:
            return
        elif idx == 1:
            self.assert_(False, PoolTestCase.MotorSimulatorAlreadyRunning)
        elif idx == 2:
            self.assert_(False, "Motor Simulator terminated unexpectedly")
        elif idx == 3:
            self.assert_(False, "Motor Simulator startup time exceeded")

    def stopMotorSimulator(self):
        """Stops the Motor Simulator"""

        if not self.needsMotorSimulator():
            return

        if self.motsim_process is None:
            return

        if not self.motsim_process.isalive():
            return

        self.motsim_process.terminate()

        if self.motsim_process.isalive():
            # flush any existing output
            self.motsim_process.close()
            if not self.motsim_process.isalive():
                return
        else:
            return

        self.motsim_process.terminate(force=True)
        if not self.motsim_process.isalive():
            self.assert_(
                False, "Motor Simulator hangs: Forced to do a SIGKILL")

        self.assert_(False, "Motor Simulator hangs EVEN with SIGKILL")

    def startCounterTimerSimulator(self):
        """Starts the Counter Timer Simulator"""

        if not self.needsCounterTimerSimulator():
            return

        self.ctsim_process = pexpect.spawn("python",
                                           args=[self.ctsim_exec,
                                                 self.ctsim_ds_instance])

        idx = self.ctsim_process.expect([PoolTestCase.CounterTimerSimulatorReadyMsg,
                                         PoolTestCase.CounterTimerSimulatorAlreadyRunning,
                                         pexpect.EOF,
                                         pexpect.TIMEOUT],
                                        timeout=PoolTestCase.CounterTimerSimulatorMaxStartupTime)

        if idx == 0:
            return
        elif idx == 1:
            self.assert_(False, PoolTestCase.CounterTimerAlreadyRunning)
        elif idx == 2:
            self.assert_(
                False, "Counter Timer Simulator terminated unexpectedly")
        elif idx == 3:
            self.assert_(
                False, "Counter Timer Simulator startup time exceeded")

    def stopCounterTimerSimulator(self):
        """Stops the Counter Timer Simulator"""

        if not self.needsCounterTimerSimulator():
            return

        if self.ctsim_process is None:
            return

        if not self.ctsim_process.isalive():
            return

        self.ctsim_process.terminate()

        if self.ctsim_process.isalive():
            # flush any existing output
            self.ctsim_process.close()
            if not self.ctsim_process.isalive():
                return
        else:
            return

        self.ctsim_process.terminate(force=True)
        if not self.ctsim_process.isalive():
            self.assert_(
                False, "Counter Timer Simulator hangs: Forced to do a SIGKILL")

        self.assert_(False, "Counter Timer Simulator hangs EVEN with SIGKILL")

    def startPool_PopenStyle(self):
        """Starts the Device pool"""

        try:
            self.pool_process = subprocess.Popen([self.pool_exec,
                                                  self.pool_ds_instance],
                                                 stdout=subprocess.PIPE,
                                                 stderr=subprocess.STDOUT)
        except OSError as e:
            if e.strerror == PoolTestCase.PoolExecNotFound:
                self.assert_(
                    False, "Could not find Pool executable. Make sure it is in the PATH")
            else:
                raise

        self.failIf(self.pool_process.stdout is None,
                    "Was not able to grab Pool output")

        self.pool_start_time = time.time()
        while True:
            delta_time = time.time() - self.pool_start_time

            self.failIf(delta_time >= PoolTestCase.PoolMaxStartupTime,
                        "Pool startup time exceeded")

            line = self.pool_process.stdout.readline()

            if line.count(PoolTestCase.PoolReadyMsg) > 0:
                break

            self.failIfEqual(line, PoolTestCase.PoolAlreadyRunning,
                             PoolTestCase.PoolAlreadyRunning)

    def stopPool_PopenStyle(self):
        """Stops the Device Pool"""

        self.pool_stop_time = time.time()
        if self.pool_process is None:
            return

        if self.pool_process.poll() is None:
            os.kill(self.pool_process.pid, signal.SIGINT)

        while self.pool_process.poll() is None:
            delta_time = time.time() - self.pool_stop_time

            if delta_time > PoolTestCase.PoolMaxShutdownTime:
                os.kill(self.pool_process.pid, signal.SIGKILL)
                self.assert_(False, "Pool hangs: Forced to do a kill -9")

            time.sleep(0.2)

        self.failUnlessEqual(self.pool_process.returncode, 0,
                             "Pool exited with code %d" % self.pool_process.returncode)
        self.pool_process = None

    def startMotorSimulator_PopenStyle(self):
        """Starts the Motor Simulator"""
        self.motsim_process = subprocess.Popen(["python",
                                                self.motsim_exec,
                                                self.motsim_ds_instance],
                                               stdout=subprocess.PIPE,
                                               stderr=subprocess.STDOUT,
                                               bufsize=0)

        self.failIf(self.motsim_process.stdout is None,
                    "Was not able to grab Motor Simulator output")

        self.motsim_start_time = time.time()
        while True:
            delta_time = time.time() - self.motsim_start_time

            self.failIf(delta_time >= PoolTestCase.MotorSimulatorMaxStartupTime,
                        "Motor Simulator startup time exceeded")

            line = self.motsim_process.stdout.readline()
            sys.stderr.write(line)
            sys.stderr.flush()

            if line.count(PoolTestCase.MotorSimulatorReadyMsg) > 0:
                break

            self.failIfEqual(line, PoolTestCase.MotorSimulatorAlreadyRunning,
                             PoolTestCase.MotorSimulatorAlreadyRunning)

    def stopMotorSimulator_PopenStyle(self):
        """Stops the Motor Simulator"""

        self.motsim_stop_time = time.time()
        if self.motsim_process is None:
            return

        os.kill(self.motsim_process.pid, signal.SIGINT)

        while self.motsim_process.poll() is None:
            delta_time = time.time() - self.motsim_stop_time

            if delta_time > PoolTestCase.MotorSimulatorMaxShutdownTime:
                os.kill(self.motsim_process.pid, signal.SIGKILL)
                self.assert_(
                    False, "MotorSimulator hangs: Forced to do a kill -9")

            time.sleep(0.2)

        self.failUnlessEqual(self.motsim_process.returncode, 0,
                             "MotorSimulator exited with code %d" % self.motsim_process.returncode)
        self.motsim_process = None

    def startCounterTimerSimulator_PopenStyle(self):
        """Starts the CounterTimer Simulator"""

        self.ctsim_process = subprocess.Popen(["python",
                                               self.ctsim_exec,
                                               self.ctsim_ds_instance],
                                              stdout=subprocess.PIPE,
                                              stderr=subprocess.STDOUT)

        self.failIf(self.ctsim_process.stdout is None,
                    "Was not able to grab CounterTimer Simulator output")

        self.ctsim_start_time = time.time()
        while True:
            delta_time = time.time() - self.ctsim_start_time

            self.failIf(delta_time >= PoolTestCase.CounterTimerSimulatorMaxStartupTime,
                        "CounterTimer Simulator startup time exceeded")

            line = self.ctsim_process.stdout.readline()

            if line.count(PoolTestCase.CounterTimerSimulatorReadyMsg) > 0:
                break

        self.failIfEqual(line, PoolTestCase.CounterTimerSimulatorAlreadyRunning,
                         PoolTestCase.CounterTimerSimulatorAlreadyRunning)
        self.motsim_process = None

    def stopCounterTimerSimulator_PopenStyle(self):
        """Stops the CounterTimer Simulator"""

        self.ctsim_stop_time = time.time()
        if self.ctsim_process is None:
            return

        os.kill(self.ctsim_process.pid, signal.SIGINT)

        while self.ctsim_process.poll() is None:
            delta_time = time.time() - self.ctsim_stop_time

            if delta_time > PoolTestCase.CounterTimerSimulatorMaxShutdownTime:
                os.kill(self.ctsim_process.pid, signal.SIGKILL)
                self.assert_(
                    False, "CounterTimerSimulator hangs: Forced to do a kill -9")

            time.sleep(0.2)

        self.failUnlessEqual(self.ctsim_process.returncode, 0,
                             "MotorSimulator exited with code %d" % self.ctsim_process.returncode)
        self.ctsim_process = None


class PoolTestResult(unittest.TestResult):
    """A test result class that can print formatted text results to a stream.
    """

    style = 'test'
    descStyle = 'testDescription'
    startedStyle = 'testStarted'
    successStyle = 'testSuccess'
    errorStyle = 'testError'
    failureStyle = 'testFailure'

    def __init__(self, stream, descriptions, verbosity):
        unittest.TestResult.__init__(self)
        self.stream = stream
        self.showAll = verbosity > 1
        self.dots = verbosity == 1
        self.descriptions = descriptions
        self.testsSucceeded = 0

    def getDescription(self, test):
        if self.descriptions:
            return test.shortDescription() or str(test)
        else:
            return str(test)

    def startTest(self, test):
        unittest.TestResult.startTest(self, test)
        s = self.stream
        desc = self.getDescription(test)
        descId = "desc%d" % self.testsRun
        statusId = "status%d" % self.testsRun
        s.write("<TR class='%s'>"
                "<TD id='%s' class='%s'>%s</TD>"
                "<TD id='%s' class='%s'>STARTED</TD>" %
                (self.style, descId, self.descStyle, desc,
                 statusId, self.startedStyle))
        s.flush()

    def _addFinished(self):
        s = self.stream
        s.seek(-25, 1)
        s.write("testFinished'>FINISHED</TD>")
        s.flush()

    def addSuccess(self, test):
        unittest.TestResult.addSuccess(self, test)
        self.testsSucceeded += 1
        s = self.stream
        resultId = "result%d" % self.testsRun
        self._addFinished()
        s.write("<TD id='%s' class='%s'>ok</TD>\n" %
                (resultId, self.successStyle))
        s.flush()

    def addError(self, test, err):
        unittest.TestResult.addError(self, test, err)
        s = self.stream
        resultId = "result%d" % self.testsRun
        self._addFinished()
        s.write("<TD id='%s' class='%s'>ERROR</TD>\n" %
                (resultId, self.errorStyle))
        s.flush()

    def addFailure(self, test, err):
        unittest.TestResult.addFailure(self, test, err)
        s = self.stream
        resultId = "result%d" % self.testsRun
        self._addFinished()
        s.write("<TD id='%s' class='%s'>FAIL</TD>\n" %
                (resultId, self.failureStyle))
        s.flush()

    def printErrors(self):
        s = self.stream
        s.write("<p><table class='test'>")
        s.write("<caption class='test'>Errors & Failures details</caption>")
        self.printErrorList("Errors", self.errors)
        self.printErrorList("Failures", self.failures)
        s.write("</table></p>")

    def printErrorList(self, flavour, errors):
        s = self.stream
        s.write("<tr><td class='summary%s'>%s</td></tr>" % (flavour, flavour))
        for test, err in errors:
            s.write("<tr><td class='summaryDescription'>%s</td></tr>" %
                    self.getDescription(test))
            err = err.replace('<', '&lt;').replace(
                '>', '&gt;').replace('\n', '<br/>')
            s.write("<tr><td class='summaryDetails'>%s</td></tr>" % err)


class PoolTestSuite(unittest.TestSuite):
    """A Pool Test Suite"""
    pass


class PoolTestLoader(unittest.TestLoader):
    """A Pool Test Loader"""
    pass


class PoolTestRunner(unittest.TextTestRunner):
    """A test runner class that displays results HTML.
    """
    css = "test.css"

    def _makeResult(self):
        return PoolTestResult(self.stream, self.descriptions, self.verbosity)

    def printHeader(self, t):
        s = self.stream
        time_str = time.ctime(t)
        s.write("<html><head><title>Pool test</title>\n"
                "<meta http-equiv='refresh' content='2'>\n"
                "<link rel='stylesheet' href='%s' type='text/css'/>\n"
                "</head>\n<body>"
                "<p><table class='info'>"
                "<caption class='test'>Environment Summary</caption>"
                "<tr><td>Start time:</td><td>%s</td></tr>"
                "<tr><td>User name:</td><td>%s</td></tr>"
                "<tr><td>uname:</td><td>%s</td></tr>"
                "<tr><td>python:</td><td>%s</td></tr>"
                "<tr><td>PyTango:</td><td>%s (%s)</td></tr>"
                "</table></p>"
                "<p><table class='test'>"
                "<caption class='test'>Test Summary</caption>\n"
                "<tr><th>Test</th><th width=100>Status</th><th width=100>Result</th></tr>\n" %
                (self.css, time_str, os.getlogin(), str(os.uname()),
                 sys.version, PyTango.Release.version, PyTango.Release.version_description))

        s.flush()

    def run(self, test):
        "Run the given test case or test suite."

        result = self._makeResult()

        startTime = time.time()
        self.printHeader(startTime)
        test(result)
        stopTime = time.time()

        timeTaken = stopTime - startTime

        # result.printErrors()
        # self.stream.writeln(result.separator2)
        run = result.testsRun

        self.stream.write("<tr><td/><td class='testSummary' colspan='2'>"
                          "Ran %d test%s in %.3fs</td></tr>" %
                          (run, run != 1 and "s" or "", timeTaken))

        failed, errored = map(len, (result.failures, result.errors))
        succeeded = result.testsSucceeded

        self.stream.write("<tr><td/><td colspan='2'>"
                          "<table><tr>"
                          "<td class='testsFailed'>%03d FAILED</td>"
                          "<td class='testsError'>%03d ERRORS</td>"
                          "<td class='testsSucceeded'>%03d SUCCEEDED</td>"
                          "</tr></table>\n" % (failed, errored, succeeded))
        self.stream.write("</table></p>")

        result.printErrors()

        self.stream.write("</body></html>")
        self.stream.flush()
        return result


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MSEditor.ui'
#
# Created: Fri Nov 19 12:49:34 2010
#      by: PyQt4 UI code generator 4.4.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui


class Ui_MSEditor(object):

    def setupUi(self, MSEditor):
        MSEditor.setObjectName("MSEditor")
        MSEditor.resize(400, 429)
        self.gridLayout = QtGui.QGridLayout(MSEditor)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.instanceNameLineEdit = QtGui.QLineEdit(MSEditor)
        self.instanceNameLineEdit.setObjectName("instanceNameLineEdit")
        self.gridLayout_2.addWidget(self.instanceNameLineEdit, 0, 1, 1, 1)
        self.instanceNameLabel = QtGui.QLabel(MSEditor)
        self.instanceNameLabel.setObjectName("instanceNameLabel")
        self.gridLayout_2.addWidget(self.instanceNameLabel, 0, 0, 1, 1)
        self.msDeviceNameLineEdit = QtGui.QLineEdit(MSEditor)
        self.msDeviceNameLineEdit.setObjectName("msDeviceNameLineEdit")
        self.gridLayout_2.addWidget(self.msDeviceNameLineEdit, 2, 1, 1, 1)
        self.msAliasLineEdit = QtGui.QLineEdit(MSEditor)
        self.msAliasLineEdit.setObjectName("msAliasLineEdit")
        self.gridLayout_2.addWidget(self.msAliasLineEdit, 3, 1, 1, 1)
        self.msVersionLineEdit = QtGui.QLineEdit(MSEditor)
        self.msVersionLineEdit.setObjectName("msVersionLineEdit")
        self.gridLayout_2.addWidget(self.msVersionLineEdit, 4, 1, 1, 1)
        self.msDeviceNameLabel = QtGui.QLabel(MSEditor)
        self.msDeviceNameLabel.setObjectName("msDeviceNameLabel")
        self.gridLayout_2.addWidget(self.msDeviceNameLabel, 2, 0, 1, 1)
        self.msAliasLabel = QtGui.QLabel(MSEditor)
        self.msAliasLabel.setObjectName("msAliasLabel")
        self.gridLayout_2.addWidget(self.msAliasLabel, 3, 0, 1, 1)
        self.msVersionLabel = QtGui.QLabel(MSEditor)
        self.msVersionLabel.setObjectName("msVersionLabel")
        self.gridLayout_2.addWidget(self.msVersionLabel, 4, 0, 1, 1)
        self.msDeviceNameCheckBox = QtGui.QCheckBox(MSEditor)
        self.msDeviceNameCheckBox.setObjectName("msDeviceNameCheckBox")
        self.gridLayout_2.addWidget(self.msDeviceNameCheckBox, 2, 2, 1, 1)
        self.msAliasCheckBox = QtGui.QCheckBox(MSEditor)
        self.msAliasCheckBox.setObjectName("msAliasCheckBox")
        self.gridLayout_2.addWidget(self.msAliasCheckBox, 3, 2, 1, 1)
        self.msVersionCheckBox = QtGui.QCheckBox(MSEditor)
        self.msVersionCheckBox.setObjectName("msVersionCheckBox")
        self.gridLayout_2.addWidget(self.msVersionCheckBox, 4, 2, 1, 1)
        self.poolNameComboBox = QtGui.QComboBox(MSEditor)
        self.poolNameComboBox.setObjectName("poolNameComboBox")
        self.gridLayout_2.addWidget(self.poolNameComboBox, 1, 1, 1, 1)
        self.poolNameLabel = QtGui.QLabel(MSEditor)
        self.poolNameLabel.setObjectName("poolNameLabel")
        self.gridLayout_2.addWidget(self.poolNameLabel, 1, 0, 1, 1)
        self.doorNameLabel = QtGui.QLabel(MSEditor)
        self.doorNameLabel.setObjectName("doorNameLabel")
        self.gridLayout_2.addWidget(self.doorNameLabel, 5, 0, 1, 1)
        self.doorAliasLabel = QtGui.QLabel(MSEditor)
        self.doorAliasLabel.setObjectName("doorAliasLabel")
        self.gridLayout_2.addWidget(self.doorAliasLabel, 6, 0, 1, 1)
        self.doorNameCheckBox = QtGui.QCheckBox(MSEditor)
        self.doorNameCheckBox.setObjectName("doorNameCheckBox")
        self.gridLayout_2.addWidget(self.doorNameCheckBox, 5, 2, 1, 1)
        self.doorAliasCheckBox = QtGui.QCheckBox(MSEditor)
        self.doorAliasCheckBox.setObjectName("doorAliasCheckBox")
        self.gridLayout_2.addWidget(self.doorAliasCheckBox, 6, 2, 1, 1)
        self.doorNameLineEdit = QtGui.QLineEdit(MSEditor)
        self.doorNameLineEdit.setObjectName("doorNameLineEdit")
        self.gridLayout_2.addWidget(self.doorNameLineEdit, 5, 1, 1, 1)
        self.doorAliasLineEdit = QtGui.QLineEdit(MSEditor)
        self.doorAliasLineEdit.setObjectName("doorAliasLineEdit")
        self.gridLayout_2.addWidget(self.doorAliasLineEdit, 6, 1, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        self.groupBox = QtGui.QGroupBox(MSEditor)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtGui.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.msPathList = QtGui.QListWidget(self.groupBox)
        self.msPathList.setMouseTracking(True)
        self.msPathList.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.msPathList.setSelectionBehavior(
            QtGui.QAbstractItemView.SelectItems)
        self.msPathList.setObjectName("msPathList")
        self.horizontalLayout.addWidget(self.msPathList)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.addButton = QtGui.QToolButton(self.groupBox)
        self.addButton.setObjectName("addButton")
        self.verticalLayout.addWidget(self.addButton)
        self.removeButton = QtGui.QToolButton(self.groupBox)
        self.removeButton.setObjectName("removeButton")
        self.verticalLayout.addWidget(self.removeButton)
        self.upButton = QtGui.QToolButton(self.groupBox)
        self.upButton.setObjectName("upButton")
        self.verticalLayout.addWidget(self.upButton)
        self.downButton = QtGui.QToolButton(self.groupBox)
        self.downButton.setObjectName("downButton")
        self.verticalLayout.addWidget(self.downButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.gridLayout.addWidget(self.groupBox, 1, 0, 1, 1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtGui.QSpacerItem(
            40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.createButton = QtGui.QPushButton(MSEditor)
        self.createButton.setObjectName("createButton")
        self.horizontalLayout_2.addWidget(self.createButton)
        self.closeButton = QtGui.QPushButton(MSEditor)
        self.closeButton.setObjectName("closeButton")
        self.horizontalLayout_2.addWidget(self.closeButton)
        self.gridLayout.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)

        self.retranslateUi(MSEditor)
        QtCore.QObject.connect(self.msDeviceNameCheckBox, QtCore.SIGNAL(
            "toggled(bool)"), self.msDeviceNameLineEdit.setDisabled)
        QtCore.QObject.connect(self.msAliasCheckBox, QtCore.SIGNAL(
            "toggled(bool)"), self.msAliasLineEdit.setDisabled)
        QtCore.QObject.connect(self.msVersionCheckBox, QtCore.SIGNAL(
            "toggled(bool)"), self.msVersionLineEdit.setDisabled)
        QtCore.QObject.connect(
            self.closeButton, QtCore.SIGNAL("clicked()"), MSEditor.close)
        QtCore.QObject.connect(self.doorNameCheckBox, QtCore.SIGNAL(
            "toggled(bool)"), self.doorNameLineEdit.setDisabled)
        QtCore.QObject.connect(self.doorAliasCheckBox, QtCore.SIGNAL(
            "toggled(bool)"), self.doorAliasLineEdit.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(MSEditor)

    def retranslateUi(self, MSEditor):
        MSEditor.setWindowTitle(QtGui.QApplication.translate(
            "MSEditor", "Create Pool", None, QtGui.QApplication.UnicodeUTF8))
        self.instanceNameLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "Instance name", None, QtGui.QApplication.UnicodeUTF8))
        self.msDeviceNameLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "MS device name", None, QtGui.QApplication.UnicodeUTF8))
        self.msAliasLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "MS alias (optional)", None, QtGui.QApplication.UnicodeUTF8))
        self.msVersionLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "MS version", None, QtGui.QApplication.UnicodeUTF8))
        self.msDeviceNameCheckBox.setText(QtGui.QApplication.translate(
            "MSEditor", "Automatic", None, QtGui.QApplication.UnicodeUTF8))
        self.msAliasCheckBox.setText(QtGui.QApplication.translate(
            "MSEditor", "Automatic", None, QtGui.QApplication.UnicodeUTF8))
        self.msVersionCheckBox.setText(QtGui.QApplication.translate(
            "MSEditor", "Automatic", None, QtGui.QApplication.UnicodeUTF8))
        self.poolNameLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "Pool name", None, QtGui.QApplication.UnicodeUTF8))
        self.doorNameLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "Door name", None, QtGui.QApplication.UnicodeUTF8))
        self.doorAliasLabel.setText(QtGui.QApplication.translate(
            "MSEditor", "Door alias (optional)", None, QtGui.QApplication.UnicodeUTF8))
        self.doorNameCheckBox.setText(QtGui.QApplication.translate(
            "MSEditor", "Automatic", None, QtGui.QApplication.UnicodeUTF8))
        self.doorAliasCheckBox.setText(QtGui.QApplication.translate(
            "MSEditor", "Automatic", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate(
            "MSEditor", "MacroPath", None, QtGui.QApplication.UnicodeUTF8))
        self.createButton.setText(QtGui.QApplication.translate(
            "MSEditor", "Create", None, QtGui.QApplication.UnicodeUTF8))
        self.closeButton.setText(QtGui.QApplication.translate(
            "MSEditor", "Close", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    MSEditor = QtGui.QDialog()
    ui = Ui_MSEditor()
    ui.setupUi(MSEditor)
    MSEditor.show()
    sys.exit(app.exec_())



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

__all__ = ['FakePool', 'FakeElement']

from sardana.pool.poolcontrollermanager import ControllerManager


class FakePool(object):
    ''' Fake class to simulate the behavior of the Pool class
    '''
    acq_loop_sleep_time = 0.1
    acq_loop_states_per_value = 10
    motion_loop_sleep_time = 0.1
    motion_loop_states_per_position = 10
    drift_correction = True

    def __init__(self, poolpath=[], loglevel=None):
        self.ctrl_manager = ControllerManager()
        if loglevel:
            self.ctrl_manager.setLogLevel(loglevel)
        self.ctrl_manager.set_pool(self)
        self.ctrl_manager.setControllerPath(poolpath)
        self.elements = {}
        self.elements_by_full_name = {}
        self._freeId = 1

    def add_element(self, element):
        self.elements[element.id] = element
        self.elements_by_full_name[element.full_name] = element

    def get_element(self, id):
        return self.elements[id]

    def get_element_by_full_name(self, full_name):
        return self.elements_by_full_name[full_name]

    def get_free_id(self):
        while True:
            try:
                self.get_element(self._freeId)
                self._freeId += 1
            except KeyError:
                return self._freeId

    def get_free_name(self, base_name):
        num = 1
        while True:
            try:
                self.get_element_by_full_name(base_name + "%s" % num)
                num += 1
            except KeyError:
                return base_name + "%s" % num

    def get_manager(self):
        return self.ctrl_manager

    def cleanup(self):
        self.ctrl_manager.cleanUp()
        self.ctrl_manager.reInit()
        self.elements = {}
        self.elements_by_full_name = {}

# TODO: this should be a mock


class FakeElement(object):
    '''Fake pool element'''

    def __init__(self, pool, name="FakeElement"):
        self.pool = pool
        self.name = name

    def on_element_changed(self, *args, **kwargs):
        pass



##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains macros that demonstrate the usage of acquisition"""

__all__ = ["acq_meas_async"]

__docformat__ = 'restructuredtext'

from sardana.macroserver.macro import Macro, Type


class acq_meas_async(Macro):
    """A macro that executes an asynchronous acquisition of a measurement
    group. The acquisition can be cancelled by a Ctrl-C signal.

    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['meas', Type.MeasurementGroup, None, 'meas. group to be acquired'],
        ['integ_time', Type.Float, None, 'integration time'],
    ]

    def run(self, meas, integ_time):
        self.info('starting acquisition for: {0}'.format(integ_time))
        meas.putIntegrationTime(integ_time)
        meas.setNbStarts(1)
        meas.prepare()
        id_ = meas.startCount()
        try:
            # Do whatever here (while the meas is acquiring)
            self.info('state: {0}'.format(meas.getStateEG().readValue()))
            # End Do whatever here
        finally:
            meas.waitCount(id=id_)
            # this line will not be printed in case of abort (Ctrl-C)
            # due to https://github.com/sardana-org/sardana/issues/10
            self.info('acquired values: {0}'.format(meas.getValues()))


class acq_expchannel_async(Macro):
    """A macro that executes an asynchronous acquisition of an experimental
    channel. The acquisition can be cancelled by a Ctrl-C signal.

    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['channel', Type.ExpChannel, None, 'channel to be acquired'],
        ['integ_time', Type.Float, None, 'integration time'],
    ]

    def run(self, channel, integ_time):
        self.info('starting acquisition for: {0}'.format(integ_time))
        channel.putIntegrationTime(integ_time)
        id_ = channel.startCount()
        try:
            # Do whatever here (while the channel is acquiring)
            self.info('state: {0}'.format(channel.getStateEG().readValue()))
            # End Do whatever here
        finally:
            channel.waitCount(id=id_)
            # this line will not be printed in case of abort (Ctrl-C)
            # due to https://github.com/sardana-org/sardana/issues/10
            self.info('acquired value: {0}'.format(channel.getValue()))


class acq_expchannel(Macro):
    """A macro that executes an synchronous acquisition of an experimental
    channel. The acquisition can be cancelled by a Ctrl-C signal.

    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['channel', Type.ExpChannel, None, 'channel to be acquired'],
        ['integ_time', Type.Float, None, 'integration time'],
    ]

    def run(self, channel, integ_time):
        self.info('starting acquisition for: {0}'.format(integ_time))
        _, value = channel.count(integ_time)
        self.info('acquired value: {0}'.format(value))



##############################################################################
##
# This file is part of Sardana
##
# http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import os
import time
import timeit
import numpy
from threading import Event, Timer

from unittest import TestCase
from taurus.core.util import ThreadPool

from sardana.pool.pooldefs import SynchDomain, SynchParam
from sardana.sardanaevent import EventGenerator, EventType, EventReceiver
from sardana.util.funcgenerator import FunctionGenerator


configuration_negative = [{SynchParam.Initial: {SynchDomain.Position: 0.},
                           SynchParam.Delay: {SynchDomain.Time: 0.1},
                           SynchParam.Active: {SynchDomain.Position: -.1,
                                               SynchDomain.Time: .01, },
                           SynchParam.Total: {SynchDomain.Position: -.2,
                                              SynchDomain.Time: 0.1},
                           SynchParam.Repeats: 10}]

configuration_positive = [{SynchParam.Initial: {SynchDomain.Position: 0.},
                           SynchParam.Delay: {SynchDomain.Time: 0.3},
                           SynchParam.Active: {SynchDomain.Position: .1,
                                               SynchDomain.Time: .01, },
                           SynchParam.Total: {SynchDomain.Position: .2,
                                              SynchDomain.Time: .02},
                           SynchParam.Repeats: 10}]


class Position(EventGenerator):

    def __init__(self):
        EventGenerator.__init__(self)
        self.value = None
        self.error = False

    def run(self, start=0, end=2, step=0.01, sleep=0.01):
        for position in numpy.arange(start, end, step):
            self.value = position
            self.fire_event(EventType("Position"), self)
            time.sleep(sleep)


class Listener(EventReceiver):

    def __init__(self):
        EventReceiver.__init__(self)
        self.init()

    def init(self):
        self.start = False
        self.active_event_ids = list()
        self.passive_event_ids = list()
        self.end = False

    def event_received(self, *args, **kwargs):
        _, type_, value = args
        name = type_.name
        if name == "active":
            self.active_event_ids.append(value)
        elif name == "passive":
            self.passive_event_ids.append(value)
        elif name == "start":
            self.start = True
        elif name == "end":
            self.end = True
        else:
            ValueError("wrong event type")


class FuncGeneratorTestCase(TestCase):

    def setUp(self):
        self.thread_pool = ThreadPool("TestThreadPool", Psize=2)
        self.func_generator = FunctionGenerator()
        self.event = Event()
        self.listener = Listener()
        self.func_generator.add_listener(self.listener)

    def _done(self, _):
        self.event.set()
        self.event.clear()

    def test_sleep(self):
        delta = 0
        if os.name == "nt":
            delta = 0.02
        for i in [0.01, 0.13, 1.2]:
            stmt = "fg.sleep(%f)" % i
            setup = "from sardana.util.funcgenerator import FunctionGenerator;\
                     fg = FunctionGenerator()"
            period = timeit.timeit(stmt, setup, number=1)
            period_ok = i
            msg = "sleep period: %f, expected: %f +/- %f" % (period, period_ok,
                                                             delta)
            self.assertAlmostEqual(period, period_ok, delta=0.02, msg=msg)

    def test_run_time(self):
        self.func_generator.initial_domain = SynchDomain.Time
        self.func_generator.set_configuration(configuration_positive)
        self.func_generator.start()
        self.thread_pool.add(self.func_generator.run, self._done)
        self.event.wait(100)
        active_event_ids = self.listener.active_event_ids
        active_event_ids_ok = list(range(0, 10))
        msg = "Received active event ids: %s, expected: %s" % (
            active_event_ids, active_event_ids_ok)
        self.assertListEqual(active_event_ids, active_event_ids_ok, msg)
        self.assertTrue(self.listener.start, "Start event is missing")
        self.assertTrue(self.listener.end, "End event is missing")

    def test_stop_time(self):
        self.func_generator.initial_domain = SynchDomain.Time
        self.func_generator.set_configuration(configuration_positive)
        self.func_generator.start()
        self.thread_pool.add(self.func_generator.run, self._done)
        while not self.func_generator.is_running():
            time.sleep(0.01)
        # starting timer that will stop generation
        Timer(0.2, self.func_generator.stop).start()
        self.event.wait(100)
        self.assertFalse(self.func_generator.is_running(), "Stopping failed")
        self.listener.init()
        self.test_run_time()

    def test_run_position_negative(self):
        position = Position()
        position.add_listener(self.func_generator)
        self.func_generator.initial_domain = SynchDomain.Position
        self.func_generator.active_domain = SynchDomain.Position
        self.func_generator.direction = -1
        self.func_generator.set_configuration(configuration_negative)
        self.func_generator.start()
        self.thread_pool.add(self.func_generator.run, self._done)
        while not self.func_generator.is_running():
            time.sleep(0.1)
        self.thread_pool.add(position.run, None, 0, -2, -.01)
        self.event.wait(3)
        position.remove_listener(self.func_generator)
        active_event_ids = self.listener.active_event_ids
        active_event_ids_ok = list(range(0, 10))
        msg = "Received active event ids: %s, expected: %s" % (active_event_ids,
                                                               active_event_ids_ok)
        self.assertListEqual(active_event_ids, active_event_ids_ok, msg)

    def test_run_position_positive(self):
        position = Position()
        position.add_listener(self.func_generator)
        self.func_generator.initial_domain = SynchDomain.Position
        self.func_generator.active_domain = SynchDomain.Position
        self.func_generator.direction = 1
        self.func_generator.set_configuration(configuration_positive)
        self.func_generator.start()
        self.thread_pool.add(self.func_generator.run, self._done)
        while not self.func_generator.is_running():
            time.sleep(0.1)
        self.thread_pool.add(position.run, None, 0, 2, .01)
        self.event.wait(3)
        position.remove_listener(self.func_generator)
        active_event_ids = self.listener.active_event_ids
        active_event_ids_ok = list(range(0, 10))
        msg = "Received active event ids: %s, expected: %s" % (active_event_ids,
                                                               active_event_ids_ok)
        self.assertListEqual(active_event_ids, active_event_ids_ok, msg)

    def test_configuration_position(self):
        self.func_generator.initial_domain = SynchDomain.Position
        self.func_generator.active_domain = SynchDomain.Position
        self.func_generator.set_configuration(configuration_negative)
        active_events = self.func_generator.active_events
        active_events_ok = numpy.arange(0, -2, -0.2).tolist()
        msg = "Active events are wrong: %s" % active_events
        for a, b in zip(active_events, active_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)
        passive_events = self.func_generator.passive_events
        passive_events_ok = numpy.arange(-.1, -2.1, -0.2).tolist()
        msg = "Passive events are wrong: %s" % passive_events
        for a, b in zip(passive_events, passive_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)

    def test_configuration_time(self):
        self.func_generator.initial_domain = SynchDomain.Time
        self.func_generator.active_domain = SynchDomain.Time
        self.func_generator.set_configuration(configuration_positive)
        active_events = self.func_generator.active_events
        active_events_ok = numpy.arange(.3, .5, 0.02).tolist()
        msg = ("Active events mismatch, received: %s, expected: %s" %
               (active_events, active_events_ok))
        for a, b in zip(active_events, active_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)
        passive_events = self.func_generator.passive_events
        passive_events_ok = numpy.arange(.31, 0.51, 0.02).tolist()
        msg = ("Passive events mismatch, received: %s, expected: %s" %
               (passive_events, passive_events_ok))
        for a, b in zip(passive_events, passive_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)

    def test_configuration_default(self):
        self.func_generator.set_configuration(configuration_positive)
        active_events = self.func_generator.active_events
        active_events_ok = numpy.arange(0, 2, 0.2).tolist()
        msg = ("Active events mismatch, received: %s, expected: %s" %
               (active_events, active_events_ok))
        for a, b in zip(active_events, active_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)
        passive_events = self.func_generator.passive_events
        passive_events_ok = numpy.arange(.31, .51, 0.02).tolist()
        msg = ("Passive events mismatch, received: %s, expected: %s" %
               (passive_events, passive_events_ok))
        for a, b in zip(passive_events, passive_events_ok):
            self.assertAlmostEqual(a, b, 10, msg)

    def tearDown(self):
        self.func_generator.remove_listener(self.listener)



import sys
import json
import csv

import PyTango
import taurus


def checkPoolElements(pool):
    pool_dev = taurus.Device(pool)

    # GET CONTROLLER CLASSES
    pool_ctrl_classes = {}
    for info in pool_dev['ControllerClassList'].value:
        info = json.loads(info)
        ctrl_class, library_path, ctrl_type = info[
            'name'], info['file_path'], info['type']
        pool_ctrl_classes[ctrl_class] = (library_path, ctrl_type)

    # GET CONTROLLERS INFO
    pool_controllers_by_type = {'Motor': [],
                                'PseudoMotor': [],
                                'IORegister': [],
                                'CounterTimer': [],
                                'OneDExpChannel': [],
                                'TwoDExpChannel': [],
                                'PseudoCounter': [],
                                'ZeroDExpChannel': [],
                                'TriggerGate': []}

    pool_controllers = {}
    for info in pool_dev['ControllerList'].value:
        info = json.loads(info)
        ctrl_name = str(info['name'])
        ctrl_class = str(info['klass'])
        file_name = str(info['file_name'])
        try:
            ctrl_library = pool_ctrl_classes[ctrl_class][0]
        except KeyError:
            print(("#WARNING: There is no controller class %s for "
                   "controller %s" % (ctrl_class, ctrl_name)))
            continue
        ctrl_type = str(info['main_type'])
        # sardana script is not compatible with the new type CTExpChannel
        # so substitute it by the old type: CounterTimer
        if ctrl_type == 'CTExpChannel':
            ctrl_type = 'CounterTimer'
        pool_controllers_by_type[ctrl_type].append(ctrl_name)
        ctrl_properties = {}
        ctrl_elements = []
        ctrl_pool_elements = []
        ctrl_dev = taurus.Device(ctrl_name)
        for prop_name in ctrl_dev.get_property_list('*'):
            if prop_name == '__SubDevices':
                continue
            prop_value = ctrl_dev.get_property(prop_name)[prop_name]
            ctrl_properties[prop_name] = ' '.join(prop_value)
        pool_controllers[ctrl_name] = {}
        pool_controllers[ctrl_name]['type'] = ctrl_type
        pool_controllers[ctrl_name]['pool'] = pool
        pool_controllers[ctrl_name]['name'] = ctrl_name
        pool_controllers[ctrl_name]['file'] = file_name
        pool_controllers[ctrl_name]['class'] = ctrl_class
        pool_controllers[ctrl_name]['properties'] = ctrl_properties
        pool_controllers[ctrl_name]['elements'] = ctrl_elements
        pool_controllers[ctrl_name]['ctrl_pool_elements'] = ctrl_pool_elements

    pool_elements = {}
    pool_elements['ExpChannels'] = pool_dev['ExpChannelList'].value or []
    pool_elements['Motors'] = pool_dev['MotorList'].value or []
    pool_elements['IORegs'] = pool_dev['IORegisterList'].value or []
    pool_instruments = pool_dev['InstrumentList'].value or []

    pool_measurement_groups = {}
    for info in pool_dev['MeasurementGroupList'].value:
        info_splitted = json.loads(info)
        mg_name = str(info_splitted["name"])
        mg_dev = taurus.Device(mg_name)
        config = mg_dev.getAttribute('configuration').read().value
        config = json.loads(config)
        controllers = config['controllers']
        elements = {}
        for ctrl, c_data in list(controllers.items()):
            if 'units' in c_data:
                c_data = c_data['units']['0']
            for _, data in list(c_data['channels'].items()):
                index = int(data['index'])
                if ctrl == '__tango__':
                    elements[index] = data['full_name']
                else:
                    elements[index] = data['name']
        indexes = sorted(elements.keys())
        elements_ordered = [elements[i] for i in indexes]
        pool_measurement_groups[mg_name] = elements_ordered

    # print '\n'
    # print '----------------------------------------------------------------'
    # print 'ELEMENTS FOR POOL ' + pool
    # print '----------------------------------------------------------------'
    # for element_type in pool_elements.keys():
    ###     elements = pool_elements[element_type]
    # print element_type+':',len(elements)

    db = taurus.Database()
    pool_elements_detail = {}
    for element_type in list(pool_elements.keys()):
        elements = pool_elements[element_type]
        for info in elements:
            info_splitted = json.loads(info)
            alias = str(info_splitted["name"])
            dev_name = info_splitted["full_name"]
            ctrl_name = info_splitted["parent"]
            ctrl_axis = info_splitted["axis"]
            specific_element_type = info_splitted["type"]
            # sardana script is not compatible with the new type CTExpChannel
            # so substitute it by the old type: CounterTimer
            if specific_element_type == 'CTExpChannel':
                specific_element_type = 'CounterTimer'

            element_dev = taurus.Device(alias)

            pool_elements_detail[alias] = {}
            pool_elements_detail[alias]['type'] = specific_element_type
            pool_elements_detail[alias]['pool'] = pool
            pool_elements_detail[alias]['ctrl'] = ctrl_name
            pool_elements_detail[alias]['name'] = alias
            pool_elements_detail[alias]['axis'] = dev_name.rsplit('/', 1)[1]
            pool_elements_detail[alias]['instrument'] = element_dev.Instrument.split('(')[
                0]
            pool_elements_detail[alias]['attr_dicts'] = {}

            normal_name = element_dev.getNormalName()
            attrs = element_dev.get_attribute_list()
            for attr, attr_dict in db.get_device_attribute_property(
                normal_name,
                list(map(str, attrs))
            ).items():
                if len(attr_dict) > 0:
                    pool_elements_detail[alias]['attr_dicts'][attr] = attr_dict
                else:
                    if attr.lower() in ['position', 'value']:
                        print('***', specific_element_type, alias, attr,
                              'NO MEMORIZED ATTRIBUTES OR ATTRIBUTE '
                              'CONFIGURATIONS ***')

    # print '\n'
    # print '----------------------------------------------------------------'
    # print len(pool_instruments),'INSTRUMENTS FOR POOL ' + pool
    # print '----------------------------------------------------------------'
    # print pool_instruments

    # CHECK ELEMENTS WITHOUT INSTRUMENT
    for element_type in list(pool_elements.keys()):
        elements = pool_elements[element_type]
        elements_with_no_instrument = []
        for info in elements:
            info_splitted = json.loads(info)
            alias = str(info_splitted["name"])
            dev_name = info_splitted["full_name"]
            ctrl_name = info_splitted["parent"]
            ctrl_axis = info_splitted["axis"]
            specific_element_type = info_splitted["type"]

            # Add info of 'physicals needed' in pseudomotor and pseudocounter
            # controllers
            try:
                ctrl = pool_controllers[ctrl_name]
            except KeyError:
                print(("#WARNING: There is no controller %s for element %s" %
                       (ctrl_name, alias)))
                continue
            ctrl['ctrl_pool_elements'].append(alias)
            if specific_element_type in ['PseudoMotor', 'PseudoCounter']:
                ctrl['elements'] = ';'.join(info_splitted["elements"])

            element_dev = taurus.Device(alias)
            if element_dev['Instrument'].value == '':
                elements_with_no_instrument.append(alias)
        # if len(elements_with_no_instrument) > 0:
        # print '\n***',element_type,'with no
        # Instrument:',elements_with_no_instrument,'***'

    # for ctrl_type, controllers in pool_controllers_by_type.iteritems():
    # if len(controllers) == 0:
    # continue
    ###
    # print '\n'
    # print '----------------------------------------------------------------'
    # print len(controllers), ctrl_type, 'CONTROLLERS FOR POOL ' + pool
    # print '----------------------------------------------------------------'
    ###
    # for ctrl in controllers:
    # print 'Controller',ctrl,':'
    # for k, v in details.iteritems():
    # if k == 'ctrl_pool_elements':
    # print '\tElements count:\t'+str(len(v))
    # print '\t'+k+':\t'+str(v)
    ###         ###
    # pool_controllers[ctrl_name]['type'] = ctrl_type
    # pool_controllers[ctrl_name]['name'] = ctrl_name
    # pool_controllers[ctrl_name]['file'] = ctrl_library
    # pool_controllers[ctrl_name]['class'] = ctrl_class
    # pool_controllers[ctrl_name]['properties'] = ctrl_properties
    # pool_controllers[ctrl_name]['elements'] = ctrl_elements
    # pool_controllers[ctrl_name]['ctrl_pool_elements'] = ctrl_pool_elements
    ###
    ###         details = pool_controllers[ctrl]
    ###         pool_elements_summary = '('+str(len(details['ctrl_pool_elements']))+')'
    # print
    # '{type}\t{name}\t{file}\t{class}\t{properties}\t{elements}'.format(**details),
    # pool_elements_summary

    columns = ['Type', 'Pool', 'Name', 'File', 'Class', 'Properties',
               'Elements', 'MOT', 'PMOT', 'IOR', 'CT', 'PC', 'ZD']
    controllers_sheet = ''
    row = '\t'.join(columns)
    controllers_sheet += row + '\n'

    columns = ['Type', 'Pool', 'Name', 'Class']
    instruments_sheet = ''
    row = '\t'.join(columns)
    instruments_sheet += row + '\n'

    columns = ['Type', 'Pool', 'Controller', 'Name', 'DeviceName',
               'Axis', 'Instrument', 'Description', 'Attributes']
    motors_sheet = ''
    row = '\t'.join(columns)
    motors_sheet += row + '\n'

    ioregs_sheet = ''
    row = '\t'.join(columns)
    ioregs_sheet += row + '\n'

    channels_sheet = ''
    row = '\t'.join(columns)
    channels_sheet += row + '\n'

    columns = ['Type', 'Pool', 'Name', 'DeviceName', 'Channels', 'Description']
    acquisition_sheet = ''
    row = '\t'.join(columns)
    acquisition_sheet += row + '\n'

    columns = ['Pool', 'Element', 'Parameter', 'Label', 'Format', 'Min Value', 'Min Alarm', 'Min Warning',
               'Max Warning', 'Max Alarm', 'Max Value', 'Unit', 'Polling Period', 'Change Event', 'Description']
    parameters_sheet = ''
    row = '\t'.join(columns)
    parameters_sheet += row + '\n'

    for ctrl_type, controllers in pool_controllers_by_type.items():
        if len(controllers) == 0:
            continue
        for ctrl in controllers:
            ctrl_details = pool_controllers[ctrl]

            for element_type in ['Motor', 'PseudoMotor', 'IORegister', 'CounterTimer', 'PseudoCounter', 'ZeroDExpChannel']:
                ctrl_details[element_type] = 0
                if ctrl_type == element_type:
                    ctrl_details[element_type] = len(
                        ctrl_details['ctrl_pool_elements'])

            if len(ctrl_details['properties']) == 0:
                ctrl_details['properties'] = ''
            else:
                properties = []
                for k, v in ctrl_details['properties'].items():
                    properties.append(k + ':' + v)
                ctrl_details['properties'] = ';'.join(properties)

            if len(ctrl_details['elements']) == 0:
                ctrl_details['elements'] = ''
            else:
                ctrl_details['elements'] = ctrl_details[
                    'elements'].replace(',', ';')

            ctrl_row_template = '{type}\t{pool}\t{name}\t{file}\t{class}\t{properties}\t{elements}\t{Motor}\t{PseudoMotor}\t{IORegister}\t{CounterTimer}\t{PseudoCounter}\t{ZeroDExpChannel}'
            row = ctrl_row_template.format(**ctrl_details)
            controllers_sheet += row + '\n'

            for alias in ctrl_details['ctrl_pool_elements']:
                elem_details = pool_elements_detail[alias]
                elem_type = elem_details['type']
                attr_dicts = elem_details['attr_dicts']
                attribute_values = []
                for attr in list(attr_dicts.keys()):
                    attr_dict = attr_dicts[attr]
                    if '__value' in attr_dict:
                        # skip memorized values of DialPosition and Position
                        # DialPosition because it is read only attribute and the
                        # current version of sardana script would not be able to se it
                        # Position is no more a memorized attribute, but the database
                        # may still contain the __value property for it - from the
                        # previous versions of Sardana (Pool in C++)
                        if attr.lower() not in ('dialposition', 'position'):
                            attribute_values.append(
                                attr + ':' + attr_dict['__value'][0])

                    elem_params = {}
                    ['Pool', 'Element', 'Parameter', 'Label', 'Format', 'Min Value', 'Min Alarm', 'Min Warning',
                        'Max Warning', 'Max Alarm', 'Max Value', 'Unit', 'Polling Period', 'Change Event', 'Description']
                    elem_params['pool'] = pool
                    elem_params['element'] = alias
                    elem_params['parameter'] = attr
                    elem_params['label'] = attr_dict.get('label', [''])[0]
                    elem_params['format'] = attr_dict.get('format', [''])[0]
                    elem_params['min_value'] = attr_dict.get(
                        'min_value', [''])[0]
                    elem_params['min_alarm'] = attr_dict.get(
                        'min_alarm', [''])[0]
                    elem_params['min_warning'] = attr_dict.get(
                        'min_warning', [''])[0]
                    elem_params['max_warning'] = attr_dict.get(
                        'max_warning', [''])[0]
                    elem_params['max_alarm'] = attr_dict.get(
                        'max_alarm', [''])[0]
                    elem_params['max_value'] = attr_dict.get(
                        'max_value', [''])[0]
                    elem_params['unit'] = attr_dict.get('unit', [''])[0]
                    elem_params['polling'] = attr_dict.get(
                        'event_period', [''])[0]
                    elem_params['event'] = attr_dict.get('abs_change', [''])[0]

                    for k, v in elem_params.items():
                        if v != '' and k not in ['pool', 'element', 'parameter']:
                            params_row_template = '{pool}\t{element}\t{parameter}\t{label}\t{format}\t{min_value}\t{min_alarm}\t{min_warning}\t{max_warning}\t{max_alarm}\t{max_value}\t{unit}\t{polling}\t{event}'
                            row = params_row_template.format(**elem_params)
                            parameters_sheet += row + '\n'
                            break

                elem_details['attrs'] = ';'.join(attribute_values)
                elem_row_template = '{type}\t{pool}\t{ctrl}\t{name}\tAutomatic\t{axis}\t{instrument}\t\t{attrs}'
                row = elem_row_template.format(**elem_details)
                if elem_type in ['Motor', 'PseudoMotor']:
                    motors_sheet += row + '\n'
                elif elem_type == 'IORegister':
                    ioregs_sheet += row + '\n'
                else:
                    channels_sheet += row + '\n'

    instrument_row_template = '{type}\t{pool}\t{name}\t{class}'
    for info in pool_instruments:
        info = json.loads(info)
        instrument_details = {}
        instrument_details['type'] = 'Instrument'
        instrument_details['pool'] = pool
        instrument_details['name'] = info['name']
        instrument_details['class'] = info['klass']
        row = instrument_row_template.format(**instrument_details)
        instruments_sheet += row + '\n'

    acq_row_template = '{type}\t{pool}\t{name}\tAutomatic\t{channels}'
    for mg_name, mg_channels in pool_measurement_groups.items():
        mg_details = {}
        mg_details['type'] = 'MeasurementGroup'
        mg_details['pool'] = pool
        mg_details['name'] = mg_name
        mg_details['channels'] = ';'.join(mg_channels)
        row = acq_row_template.format(**mg_details)
        acquisition_sheet += row + '\n'

    print('\n' * 2)
    print('################################ CONTROLLERS '
          '################################\n' * 4)
    print('\n' * 2)
    print(controllers_sheet)
    print('\n' * 2)
    print('################################ INSTRUMENTS '
          '################################\n' * 4)
    print('\n' * 2)
    print(instruments_sheet)
    print('\n' * 2)
    print('################################    MOTORS   '
          '################################\n' * 4)
    print('\n' * 2)
    print(motors_sheet)
    print('\n' * 2)
    print('################################    IOREGS   '
          '################################\n' * 4)
    print('\n' * 2)
    print(ioregs_sheet)
    print('\n' * 2)
    print('################################   CHANNELS  '
          '################################\n' * 4)
    print('\n' * 2)
    print(channels_sheet)
    print('\n' * 2)
    print('################################ ACQUISITION '
          '################################\n' * 4)
    print('\n' * 2)
    print(acquisition_sheet)
    print('\n' * 2)
    print('################################ PARAMETERS  '
          '################################\n' * 4)
    print('\n' * 2)
    print(parameters_sheet)


if __name__ == '__main__':
    if len(sys.argv) != 2 or sys.argv[1] == '?':
        print('----------------------------------------')
        print('Invalid number of arguments.')
        print('')
        print('Example of usage:')
        print('    python get_pool_config pool')
        print('')
        print('    where pool is the device name of the pool')
        print('----------------------------------------')

    pool = sys.argv[1]
    checkPoolElements(pool)



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Module with tests for macro related utils."""

from lxml import etree

import unittest
from taurus.test import insertTest
from sardana.taurus.core.tango.sardana.macro import MacroNode
from sardana.taurus.core.tango.sardana.macro import createMacroNode
from sardana.macroserver.macro import Type
from sardana.taurus.core.tango.sardana.test import (pt3d_param_def,
                                                    pt5d_param_def,
                                                    pt7d_param_def,
                                                    pt10d_param_def,
                                                    pt12d_param_def,
                                                    pt13d_param_def,
                                                    pt14d_param_def)
# TODO: Use unittest.mock instead of this fake class.
from sardana.macroserver.mstypemanager import TypeManager
from sardana.util.parser import ParamParser


class FakeMacroServer(object):
    name = "FakeMacroServer"

macro_server = FakeMacroServer()
tm = TypeManager(macro_server)

# TODO: Move the parameter definition to res/paramdef.py module
pt8_params_def = [
    {
        "name": "m_p_pair",
        "type": [
            {
                "name": "motor",
                "type": Type.Motor,
                "default_value": None,
                "description": "motor",
            },
            {
                "name": "pos",
                "type": Type.Integer,
                "default_value": None,
                "description": "position",
            }
        ],
        "description": "pair of motor and position",
        "min": 1,
        "max": None
    }
]
pt8_params_str = "mot73 5.0 mot74 8.0"

#
pt8_xml = \
    '''<macro name="pt8">
  <paramrepeat name="m_p_pair">
    <repeat nr="1">
      <param name="motor" value="mot73"/>
      <param name="pos" value="5.0"/>
    </repeat>
    <repeat nr="2">
      <param name="motor" value="mot74"/>
      <param name="pos" value="8.0"/>
    </repeat>
  </paramrepeat>
</macro>'''


@insertTest(helper_name='verifyXML', macro_name="pt8", param_def=pt8_params_def,
            param_str=pt8_params_str, expected_xml_rep=pt8_xml)
class MacroNodeTestCase(unittest.TestCase):

    def _validateXML(self, macronode_xml, expected_xml):
        '''
        :param macronode_xml: macronode lxml.etree
        :param expected_xml:  expected lxml.etree
        '''
        expected_str = etree.tostring(expected_xml, encoding='unicode')
        macronode_str = etree.tostring(macronode_xml, encoding='unicode',
                                       pretty_print=True)
        msg = "XML encodings are not equal"
        # TODO: check why macronode_str has an extra whitespace charactger
        # at the end. strips should not be necessary
        self.assertEqual(expected_str.strip(), macronode_str.strip(), msg)

    def verifyXML(self, macro_name, param_def, param_str, expected_xml_rep):
        """
        Helper to verify the generated XML of a macroNode
        :param macro_name: (str) name of the macro
        :param param_def:   (list<dict>) macro parameters definition
        :param param_value: (list<str>) list of strins representing macro
            parameters values
        :param expected_xml_rep: "pretty print" string representation of a XML
            macroNode
        """
        param_parser = ParamParser(param_def)
        param_value = param_parser.parse(param_str)
        # Create the MacroNide with the inputs
        macronode = createMacroNode(macro_name, param_def, param_value)
        # Get the MacroNode equivalent XML tree
        macronode_xml = macronode.toXml()
        # Create a XML tree
        expected_xml = etree.fromstring(expected_xml_rep)
        # Validate the XML tree
        self._validateXML(macronode_xml, expected_xml)


#### Testing a list of elements (floats) ####
@insertTest(helper_name='verifyEncoding',
            param_def=pt3d_param_def,
            macro_params=[["1", "3", "15"]],
            expected_params_list=[["1", "3", "15"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt3d_param_def,
            macro_params=[["3", [], "4"]],
            expected_params_list=[["3", "100", "4"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt3d_param_def,
            macro_params=[[[], [], []]],
            expected_params_list=[["100", "100", "100"]]
            )
#### Testing one element (moveable) followed by a ####
############ list of elements (floats). ##############
@insertTest(helper_name='verifyEncoding',
            param_def=pt5d_param_def,
            macro_params=["mot01", ["1", "3", "15"]],
            expected_params_list=["mot01", ["1", "3", "15"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt5d_param_def,
            macro_params=[[], ["1", "3", "15"]],
            expected_params_list=["mot99", ["1", "3", "15"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt5d_param_def,
            macro_params=["mot01", [[], "3", "15"]],
            expected_params_list=["mot01", ["100", "3", "15"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt5d_param_def,
            macro_params=[[], ["1", [], "15"]],
            expected_params_list=["mot99", ["1", "100", "15"]]
            )
#### Testing a list of pairs of elements (moveable, float). ####
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[["mot01", "0"]]],
            expected_params_list=[[['mot01', "0"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[["mot01"]]],
            expected_params_list=[[['mot01', "100"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[[]]],
            expected_params_list=[[['mot99', "100"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[[[], "50"]]],
            expected_params_list=[[['mot99', "50"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[['mot99', []]]],
            expected_params_list=[[['mot99', "100"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[["mot01", "0"], ["mot02", "5"], ["mot03", "10"]]],
            expected_params_list=[[["mot01", "0"], ["mot02", "5"],
                                   ["mot03", "10"]]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt7d_param_def,
            macro_params=[[["mot01", "0"], [], ["mot03"]]],
            expected_params_list=[[["mot01", "0"], ["mot99", "100"],
                                   ["mot03", "100"]]]
            )
# Testing the outer default parameters for the whole repetition.
# This test is commented because this option is not functional at the moment.
# See ticket #427 referring to this bug.
#@insertTest(helper_name='verifyEncoding',
#            param_def=pt7d_param_def,
#            macro_params=[[[]]],
#            expected_params_list=[[['mot01', "50"]]],
#            )
#### Testing list of elements (moveables) followed by a ####
################# single parameter (float). ################
@insertTest(helper_name='verifyEncoding',
            param_def=pt10d_param_def,
            macro_params=[["1", "3", "15"], "mot01"],
            expected_params_list=[["1", "3", "15"], "mot01"]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt10d_param_def,
            macro_params=[["1", "3", []], "mot01"],
            expected_params_list=[["1", "3", "100"], "mot01"]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt10d_param_def,
            macro_params=[["1", "3", "15"], []],
            expected_params_list=[["1", "3", "15"], "mot99"]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt10d_param_def,
            macro_params=[["1", [], "15"], []],
            expected_params_list=[["1", "100", "15"], "mot99"],
            )
#### Testing lists of elements (floats), followed by another list of ####
######################### elements (moveables). #########################
@insertTest(helper_name='verifyEncoding',
            param_def=pt12d_param_def,
            macro_params=[["1", "3", "4"], ["mot1", "mot2"]],
            expected_params_list=[["1", "3", "4"], ["mot1", "mot2"]]
            )
@insertTest(helper_name='verifyEncoding',
            param_def=pt12d_param_def,
            macro_params=[["1", [], "4"], [[], "mot2"]],
            expected_params_list=[["1", "100", "4"], ["mot99", "mot2"]]
            )
########################### Testing nested paramRepeats ##################
############## Groups of motors, where each group is a motor list ########
@insertTest(helper_name='verifyEncoding',
            param_def=pt13d_param_def,
            macro_params=[[["mot2", "mot3"], ["mot4", "mot5", "mot6"]]],
            expected_params_list=[[["mot2", "mot3"], ["mot4", "mot5", "mot6"]]]
            )
########################### Testing nested paramRepeats ##################
## Groups of motors, where each group is a motor list followed by a number
@insertTest(helper_name='verifyEncoding',
            param_def=pt14d_param_def,
            macro_params=[[[["mot2", "mot3"], 4],
                           [["mot4", "mot5", "mot6"], 5]]],
            expected_params_list=[[[["mot2", "mot3"], 4],
                                   [["mot4", "mot5", "mot6"], 5]]]
            )
class ParamsTestCase(unittest.TestCase):

    def verifyEncoding(self, param_def, macro_params, expected_params_list):
        """
        Helper to verify the correct building of the parameters objects tree.
        Verify that the list is recreated correctly from the parameters
        objects tree.
        :param param_def:   (list<dict>) macro parameters definition
        :param macro_params: (list<str>) list of strings representing macro
            parameters values.
        :param expected_params_list: (list<str>) expected parameters list.
        """

        # Create the MacroNode with the inputs
        node = MacroNode(name="macro_name", params_def=param_def)
        node.fromList(macro_params)

        output_params_list = node.toList()
        output_params_list.pop(0)

        msg = ("Parameters list is not encoded/decoded correctly. \n"
               "expected: %s \n"
               "received: %s" % (expected_params_list, output_params_list))
        self.assertEqual(output_params_list, expected_params_list, msg)


class DuplicateTestCase(unittest.TestCase):

    """
    Duplicate a RepeatNode and check that it has been correctly duplicated.
    """

    def testDuplication(self):
        """
        Helper to verify the correct duplication of a RepeatNode. Duplication
        of parameters.

        ..todo:: To be more unit test the use of MacroNode class should be
        avoided. Use of RepeatParamNode and its children should be enough.
        """

        # Create the MacroNode
        node = MacroNode(name="macro_name", params_def=pt7d_param_def)
        repeat_node_params = [['mot01', '0'], ['mot02', '2'], ['mot03', '4']]
        node.fromList([repeat_node_params])

        # Number of nodes before duplication
        num_initial_nodes = len(node.child(0))

        # Duplicate one RepeatNode
        num_node_to_duplicate = 1
        node_to_duplicate = node.child(0).child(num_node_to_duplicate)
        node_to_duplicate.duplicateNode()

        # Number of nodes after duplication
        num_final_nodes = len(node.child(0))

        # We get the last element RepeatNode.
        output_params_list = node.child(0).toList()
        repeat_node = node_to_duplicate.toList()
        expected_params_list = repeat_node_params + [repeat_node]

        msg = ("Repeat Node has not been correctly duplicated\n"
               "expected parameters: %s \n"
               "received parameters: %s" % (expected_params_list,
                                            output_params_list))
        self.assertEqual(output_params_list, expected_params_list, msg)

        expected_number_of_nodes = num_initial_nodes + 1
        msg = "Number of nodes after repeat node duplication is not correct"
        self.assertEqual(num_final_nodes, expected_number_of_nodes, msg)

        for i in range(expected_number_of_nodes):
            node_num = i + 1

            expected_name = '#' + str(node_num)
            new_node_name = node.child(0).child(i).name()
            msg = ("Name of node %d is not correct\n"
                   "expected name: %s \n"
                   "received name: %s" % (node_num,
                                          expected_name,
                                          new_node_name))
            self.assertEqual(new_node_name, expected_name, msg)



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Base classes for the macroserver tests"""

__all__ = ['BaseMacroServerTestCase']

import os

import PyTango
from taurus.core.util import whichexecutable
from taurus.core.tango.starter import ProcessStarter

from sardana import sardanacustomsettings
from sardana.tango.core.util import (get_free_server, get_free_device)
from sardana.tango.macroserver.MacroServer import MacroServerClass


class BaseMacroServerTestCase(object):
    """Abstract class for macroserver DS testing.
    """
    ms_ds_name = getattr(sardanacustomsettings, 'UNITTEST_MS_DS_NAME',
                         "unittest1")
    ms_name = getattr(sardanacustomsettings, 'UNITTEST_MS_NAME',
                      "macroserver/demo1/1")
    door_name = getattr(sardanacustomsettings, 'UNITTEST_DOOR_NAME',
                        "door/demo1/1")

    def setUp(self, properties=None):
        """
        Start MacroServer DS.

        :param properties: dictionary with the macroserver properies.

        """
        try:
            db = PyTango.Database()
            # Discover the MS launcher script
            msExec = whichexecutable.whichfile("MacroServer")
            # register MS server
            ms_ds_name_base = "MacroServer/" + self.ms_ds_name
            self.ms_ds_name = get_free_server(db, ms_ds_name_base)
            self._msstarter = ProcessStarter(msExec, self.ms_ds_name)
            # register MS device
            dev_name_parts = self.ms_name.split('/')
            prefix = '/'.join(dev_name_parts[0:2])
            start_from = int(dev_name_parts[2])
            self.ms_name = get_free_device(
                db, prefix, start_from)
            self._msstarter.addNewDevice(self.ms_name, klass='MacroServer')
            # register Door device
            dev_name_parts = self.door_name.split('/')
            prefix = '/'.join(dev_name_parts[0:2])
            start_from = int(dev_name_parts[2])
            self.door_name = get_free_device(db, prefix, start_from)
            self._msstarter.addNewDevice(self.door_name, klass='Door')
            # Add properties
            if properties:
                for key, values in list(properties.items()):
                    db.put_device_property(self.ms_name,
                                           {key: values})
            # start MS server
            self._msstarter.startDs(wait_seconds=20)
            self.door = PyTango.DeviceProxy(self.door_name)
        except Exception as e:
            # force tearDown in order to eliminate the MacroServer
            print(e)
            self.tearDown()

    def tearDown(self):
        """Remove the MacroServer instance and its properties file.
        """

        self._msstarter.cleanDb(force=True)
        self._msstarter = None
        self.macroserver = None
        self.door = None

        db = PyTango.Database()
        prop = db.get_device_property(self.ms_name, "EnvironmentDb")
        ms_properties = prop["EnvironmentDb"]
        if not ms_properties:
            dft_ms_properties = os.path.join(
                MacroServerClass.DefaultEnvBaseDir,
                MacroServerClass.DefaultEnvRelDir)
            ds_inst_name = self.ms_ds_name.split("/")[1]
            ms_properties = dft_ms_properties % {
                "ds_exec_name": "MacroServer",
                "ds_inst_name": ds_inst_name}
        ms_properties = os.path.normpath(ms_properties)
        extensions = [".bak", ".dat", ".dir"]
        for ext in extensions:
            name = ms_properties + ext
            if not os.path.exists(name):
                continue
            try:
                os.remove(name)
            except Exception as e:
                msg = "Not possible to remove macroserver environment file"
                print(msg)
                print(("Details: %s" % e))


if __name__ == '__main__':
    bms = BaseMacroServerTestCase()
    bms.setUp()
    print(bms.door, bms.macroserver)
    bms.tearDown()



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import time
import unittest
from sardana.pool.poolmeasurementgroup import PoolMeasurementGroup
from sardana.pool.test import (FakePool, createPoolController,
                               createPoolMeasurementGroup,
                               createPoolCounterTimer, dummyCounterTimerConf01,
                               dummyPoolCTCtrlConf01,
                               dummyMeasurementGroupConf01)


class PoolMeasurementGroupTestCase(unittest.TestCase):
    """Class used for an acquisition done by a Measurement Group with a
    dummyCounterTimer channel. The Measurement Group runs with a freshly created
    fake Pool which does not depends on the Sardana Pool.
    """

    def setUp(self):
        """Setup:
        - Use resources for Controller, CounterTimer and MeasurementGroup
        features.
        - Create Controller, CounterTimer and MeasurementGroup.
        """
        pool = FakePool()

        pc = createPoolController(pool, dummyPoolCTCtrlConf01)
        pct = createPoolCounterTimer(pool, pc, dummyCounterTimerConf01)

        pool.add_element(pc)
        pool.add_element(pct)

        self.pmg = createPoolMeasurementGroup(
            pool, dummyMeasurementGroupConf01)
        pool.add_element(self.pmg)
        self._pct = pct  # keep a reference to use it in test_acquisition

    def test_init(self):
        """check that the PoolMeasurementGroup is correctly instantiated"""
        msg = 'PoolMeasurementGroup constructor does not create ' +\
              'PoolMeasurementGroup instance'
        self.assertIsInstance(self.pmg, PoolMeasurementGroup, msg)

    def test_acquisition(self):
        """Test acquisition using the created measurement group without
        using a Sardana pool."""
        msg = 'Pool Measurement Group does not acquire'
        integ_time = .1
        self.pmg.integration_time = integ_time
        self.pmg.prepare()
        self.pmg.start_acquisition()

        acq = self.pmg.acquisition
        # 'acquiring..'
        while acq.is_running():
            time.sleep(0.05)
        self.assertEqual(self._pct.value.value, integ_time, msg)

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        self.pmg = None
        self._pct = None



import sys
import os

from PyQt4 import QtGui, QtCore, Qt

import taurus
import wiz

from taurus.core.utils import Enumeration

PoolSelectionMode = Enumeration(
    'PoolSelectionMode', ('FromSardana', 'FromDatabase'))


class SelectPoolBasePage(wiz.SardanaBasePage):

    def __init__(self, parent=None, selectionMode=PoolSelectionMode.FromDatabase):
        wiz.SardanaBasePage.__init__(self, parent)
        self.setSelectionMode(selectionMode)

        self['pool'] = self._getPool

        self.setSubTitle('Please select the Pool instance')

        panel = self.getPanelWidget()

        layout = QtGui.QFormLayout()

        self.poolCB = QtGui.QComboBox(panel)
        self.poolCB.setDuplicatesEnabled(False)
        self.poolCB.setEditable(False)

        layout.addRow("&Pool instance", self.poolCB)

        panel.setLayout(layout)

        self.connect(self.poolCB,
                     QtCore.SIGNAL('currentIndexChanged(int)'),
                     QtCore.SIGNAL('completeChanged()'))

    def getSelectionMode(self):
        return self._selectionMode

    def setSelectionMode(self, selectionMode):
        self._selectionMode = selectionMode

    def _getPool(self):
        pool_server = str(self.poolCB.currentText())
        return pool_server

    def pool(self):
        return self.wizard()['pool']

    def db(self):
        return self.wizard()['db']

    def isComplete(self):
        idx = self.poolCB.currentIndex()
        if idx >= 0:
            self.setStatus('')
            return True
        self.setStatus('No instance selected')
        return False

    def getPoolServerList(self):
        return [s[s.index('/') + 1:] for s in self.db().get_server_list() if s.startswith('Pool/')]

    def _fillCB(self):
        self.poolCB.clear()
        mode = self.getSelectionMode()
        if mode == PoolSelectionMode.FromDatabase:
            self.poolCB.addItems(self.getPoolServerList())
        else:
            # TODO
            pass

    def initializePage(self):
        wiz.SardanaBasePage.initializePage(self)
        self._fillCB()


def t1(tg_host=None, sardana=None):

    PoolExamplePages = Enumeration('PoolExamplePages',
                                   ('IntroPage', 'TangoPage', 'SardanaPage', 'PoolPage'))

    class PoolExampleIntroPage(wiz.SardanaIntroBasePage):

        def setNextPageId(self, id):
            self._nextPageId = id

        def nextId(self):
            return self._nextPageId

    from tango_host_page import SelectTangoHostBasePage

    class SelectTangoHostExamplePage(SelectTangoHostBasePage):

        def setNextPageId(self, id):
            self._nextPageId = id

        def nextId(self):
            return self._nextPageId

    from sardana_page import SelectSardanaBasePage

    class SelectSardanaExamplePage(SelectSardanaBasePage):

        def setNextPageId(self, id):
            self._nextPageId = id

        def nextId(self):
            return self._nextPageId

    app = QtGui.QApplication([])
    QtCore.QResource.registerResource(wiz.get_resources())

    w = wiz.SardanaBaseWizard()

    intro = PoolExampleIntroPage()
    w.setPage(PoolExamplePages.IntroPage, intro)

    curr_page = intro
    if tg_host is None:
        curr_page.setNextPageId(PoolExamplePages.TangoPage)
        tg_host_page = SelectTangoHostExamplePage()
        w.setPage(PoolExamplePages.TangoPage, tg_host_page)
        curr_page = tg_host_page
    else:
        w['db'] = lambda: tau.Database(tg_host)

    if sardana is None:
        curr_page.setNextPageId(PoolExamplePages.SardanaPage)
        sardana_page = SelectSardanaExamplePage()
        w.setPage(PoolExamplePages.SardanaPage, sardana_page)
        curr_page = sardana_page
    else:
        w['sardana'] = lambda: sardana

    curr_page.setNextPageId(PoolExamplePages.PoolPage)

    pool_page = SelectPoolBasePage(selectionMode=PoolSelectionMode.FromSardana)

    w.setPage(PoolExamplePages.PoolPage, pool_page)

    w.show()
    sys.exit(app.exec_())


def main():
    tg_host, sardana = None, None
    if len(sys.argv) == 2:
        tg_host = sys.argv[1]
    if len(sys.argv) > 2:
        sardana = sys.argv[2]
    t1(tg_host=tg_host, sardana=sardana)

if __name__ == '__main__':
    main()



from PyQt4 import QtCore, QtGui, Qt
import sys
from types import *
import taurus.qt.qtgui.resource
from abc import abstractmethod
import copy

############## FAKE ######################################


class PropertyInfo(object):

    def __init__(self, name, type, format, default_value=None):
        self._name = name
        self._type = type
        self._format = format
        self._default_value = default_value

    def get_name(self):
        return self._name

    def get_type(self):
        return self._type

    def get_format(self):
        return self._format

    def get_default_value(self):
        return self._default_value

#######################################################


########################## BASIC INPUT WIDGETS #################

class InputWidget (object):

    @abstractmethod
    def setValue(self, value, undo=False):
        pass

    @abstractmethod
    def getValue(self):
        return None

    @abstractmethod
    def valueChanged(self):
        pass

    def deselectWidget(self):
        pass


class BooleanWidget(QtGui.QWidget, InputWidget):

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self._formLayout = QtGui.QHBoxLayout(self)
        self.trueButton = QtGui.QRadioButton(self)
        self._formLayout.addWidget(self.trueButton)
        self.falseButton = QtGui.QRadioButton(self)
        self._formLayout.addWidget(self.falseButton)
        self.trueButton.setText("True")
        self.falseButton.setText("False")
        QtCore.QObject.connect(self.trueButton, QtCore.SIGNAL(
            "clicked()"), self.valueChanged)
        QtCore.QObject.connect(self.falseButton, QtCore.SIGNAL(
            "clicked()"), self.valueChanged)
        self.setValue(self.getDefaultValue(), undo=False)

    def valueChanged(self):
        if not (self.trueButton.isChecked() == self._actualValue):
            self.emit(QtCore.SIGNAL("valueChanged"),
                      self._actualValue, not self._actualValue)
        self._actualValue = self.trueButton.isChecked()

    def setValue(self, value, undo=False):
        if value is None:
            value = self.getDefaultValue()
        self.trueButton.setChecked(value)
        self.falseButton.setChecked(not value)
        self._actualValue = value

    def getValue(self):
        return self.trueButton.isChecked()

    @classmethod
    def getDefaultValue(self):
        return True


class BooleanComboWidget(QtGui.QComboBox, InputWidget):

    def __init__(self, parent=None):
        QtGui.QComboBox.__init__(self, parent)
        self.addItems(["True", "False"])
        self.setValue(self.getDefaultValue(), undo=False)

    def setValue(self, value, undo=False):
        if value is None:
            value = self.getDefaultValue()

        if value is True:
            self.setCurrentIndex(0)
        else:
            self.setCurrentIndex(1)

    def getValue(self):
        return not self.currentIndex()

    def valueChanged(self):
        pass

    @classmethod
    def getDefaultValue(self):
        return True


class IntegerWidget(QtGui.QLineEdit, InputWidget):

    def __init__(self, parent=None):
        QtGui.QLineEdit.__init__(self, parent)
        self.setValidator(QtGui.QIntValidator(self))
        self.setValue(self.getDefaultValue(), undo=False)

    def setValue(self, value, undo=False):
        if value is None:
            value = self.getDefaultValue()
        self.setText(str(value))
        self._actualValue = value

    def getValue(self):
        try:
            return int(self.text())
        except:
            return self.getDefaultValue()

    def focusOutEvent(self, event):  # QFocusEvent
        QtGui.QLineEdit.focusOutEvent(self, event)
        self.valueChanged()

    def valueChanged(self):
        if not (self.getValue() == self._actualValue):
            self.emit(QtCore.SIGNAL("valueChanged"),
                      self._actualValue, self.getValue())
        self._actualValue = self.getValue()
        self.setValue(self.getValue(), undo=False)  # if value is not valid

    @classmethod
    def getDefaultValue(self):
        return 0


class FloatWidget(QtGui.QLineEdit, InputWidget):

    def __init__(self, parent=None):
        QtGui.QLineEdit.__init__(self, parent)
        self.setValidator(QtGui.QDoubleValidator(self))
        self.setValue(self.getDefaultValue(), undo=False)

    def setValue(self, value, undo=False):
        if (value is None):
            value = self.getDefaultValue()
        self.setText(str(value))
        self._actualValue = value

    def getValue(self):
        try:
            return float(self.text())
        except:
            return self.getDefaultValue()

    def focusOutEvent(self, event):  # QFocusEvent
        QtGui.QLineEdit.focusOutEvent(self, event)
        self.valueChanged()

    def valueChanged(self):
        if not (self.getValue() == self._actualValue):
            self.emit(QtCore.SIGNAL("valueChanged"),
                      self._actualValue, self.getValue())
        self._actualValue = self.getValue()
        self.setValue(self.getValue(), undo=False)  # if value is not valid

    @classmethod
    def getDefaultValue(self):
        return 0.0


class StringWidget(QtGui.QLineEdit, InputWidget):

    def __init__(self, parent=None):
        QtGui.QLineEdit.__init__(self, parent)
        self.setValue(self.getDefaultValue(), undo=False)

    def setValue(self, value, undo=False):
        if value is None:
            value = self.getDefaultValue()
        self.setText(str(value))
        self._actualValue = value

    def getValue(self):
        return str(self.text())

    def textChanged(self, string):
        QtGui.QLineEdit.textChanged(self, string)

    def focusOutEvent(self, event):  # QFocusEvent
        QtGui.QLineEdit.focusOutEvent(self, event)
        self.valueChanged()

    def valueChanged(self):
        if not (self.getValue() == self._actualValue):
            self.emit(QtCore.SIGNAL("valueChanged"),
                      self._actualValue, self.getValue())
        self._actualValue = self.getValue()
        self.setValue(self.getValue(), undo=False)  # if value is not valid

    @classmethod
    def getDefaultValue(self):
        return ""

############################## TABLE WIDGET ############################


class TableWidget (QtGui.QWidget):

    def __init__(self, value, format, type, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self._actualValue = self.getDefaultValue()
        self._layout = QtGui.QHBoxLayout(self)
        self.setLayout(self._layout)
        self._tableView = QtGui.QTableView()
        self._tableView.mousePressEvent = self.mousePressEvent
        self._tableView.setSelectionMode(QtGui.QTableView.SingleSelection)
        self._layout.addWidget(self._tableView)
        self._format = format
        self._type = type
        self._verticalLayout = QtGui.QVBoxLayout()
        self._addRowButton = QtGui.QPushButton(self)
        self._addRowButton.setIcon(
            taurus.qt.qtgui.resource.getThemeIcon("list-add"))
        self._addRowButton.setText("Add Row       ")
        self._verticalLayout.addWidget(self._addRowButton)
        self._removeRowButton = QtGui.QPushButton(self)
        self._removeRowButton.setIcon(
            taurus.qt.qtgui.resource.getThemeIcon("list-remove"))
        self._removeRowButton.setText("Remove Row    ")
        self._verticalLayout.addWidget(self._removeRowButton)
        self._addColumnButton = QtGui.QPushButton(self)
        self._addColumnButton.setIcon(
            taurus.qt.qtgui.resource.getThemeIcon("list-add"))
        self._addColumnButton.setText("Add Column")
        self._verticalLayout.addWidget(self._addColumnButton)
        self._removeColumnButton = QtGui.QPushButton(self)
        self._removeColumnButton.setIcon(
            taurus.qt.qtgui.resource.getThemeIcon("list-remove"))
        self._removeColumnButton.setText("Remove Column")
        self._verticalLayout.addWidget(self._removeColumnButton)
        self._upButton = QtGui.QPushButton(self)
        self._upButton.setIcon(taurus.qt.qtgui.resource.getThemeIcon("go-up"))
        self._upButton.setText("Move Up   ")
        self._verticalLayout.addWidget(self._upButton)
        self._downButton = QtGui.QPushButton(self)
        self._downButton.setIcon(
            taurus.qt.qtgui.resource.getThemeIcon("go-down"))
        self._downButton.setText("Move Down")
        self._verticalLayout.addWidget(self._downButton)
        spacerItem = QtGui.QSpacerItem(
            20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self._verticalLayout.addItem(spacerItem)

        QtCore.QObject.connect(
            self._addRowButton, QtCore.SIGNAL("clicked()"), self._addRow)
        QtCore.QObject.connect(self._addColumnButton,
                               QtCore.SIGNAL("clicked()"), self._addColumn)
        QtCore.QObject.connect(self._removeRowButton,
                               QtCore.SIGNAL("clicked()"), self._removeRow)
        QtCore.QObject.connect(self._removeColumnButton,
                               QtCore.SIGNAL("clicked()"), self._removeColumn)
        QtCore.QObject.connect(
            self._upButton, QtCore.SIGNAL("clicked()"), self._moveUp)
        QtCore.QObject.connect(
            self._downButton, QtCore.SIGNAL("clicked()"), self._moveDown)
        self._layout.addLayout(self._verticalLayout)

        if self._type == "boolean":
            self._delegate = BooleanTableDelegate(self._tableView)
        elif self._type == "integer":
            self._delegate = IntegerTableDelegate(self._tableView)
        elif self._type == "float":
            self._delegate = FloatTableDelegate(self._tableView)
        else:
            self._delegate = StringTableDelegate(self._tableView)

        QtCore.QObject.connect(self._delegate, QtCore.SIGNAL(
            "editorValueChanged"), self._valueChanged)
        self._tableView.setItemDelegate(self._delegate)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.MinimumExpanding)
        self._tableView.setSizePolicy(sizePolicy)
        self._tableView.horizontalHeader().setDefaultSectionSize(80)
        if self._format == "1D":
            self._tableView.horizontalHeader().setVisible(False)
            self._tableView.horizontalHeader().setStretchLastSection(True)
            self._tableView.setMinimumSize(QtCore.QSize(150, 150))
            self._addColumnButton.hide()
            self._removeColumnButton.hide()
        elif self._format == "2D":
            self._tableView.setMinimumSize(QtCore.QSize(150, 250))
            self._upButton.hide()
            self._downButton.hide()

    def mousePressEvent(self, event):
        if (event.button() == QtCore.Qt.LeftButton):
            index = self._tableView.indexAt(event.pos())
            if (index.isValid()):
                if index in self._tableView.selectedIndexes():
                    QtGui.QTableView.mousePressEvent(self._tableView, event)
                    self._tableView.clearSelection()
                else:
                    self.emit(Qt.SIGNAL("selected"))
                    QtGui.QTableView.mousePressEvent(self._tableView, event)
            else:
                self._tableView.clearSelection()
        else:
            QtGui.QTableView.mousePressEvent(self._tableView, event)

    def deselectWidget(self):
        self._tableView.clearSelection()

    def _addRow(self):
        value = self.getValue()  # stored table
        # default value for every new item in the table
        defaultCellsValue = self._delegate.getDefaultValue()

        rows = len(value)
        if (self._format == "2D"):
            if (rows > 0):
                columns = len(value[0])
            else:
                columns = 1

        rowIndex = self._getSelectedIndex()[0]

        if rowIndex is None:
            rowIndex = rows

        if (self._format == "1D"):
            value.insert(rowIndex, defaultCellsValue)

        if (self._format == "2D"):
            row = []
            for i in range(columns):
                row.append(defaultCellsValue)
            value.insert(rowIndex, row)

        self.setValue(value, undo=True)

    def _getSelectedIndex(self):
        if len(self._tableView.selectedIndexes()):
            row = self._tableView.selectedIndexes()[0].row()
            column = self._tableView.selectedIndexes()[0].column()
        else:
            row = None
            column = None

        return [row, column]

    def _removeRow(self):
        rowIndex = self._getSelectedIndex()[0]
        if rowIndex is not None:
            msgBox = QtGui.QMessageBox()
            msgBox.setText("Confirmation              ")
            msgBox.setStandardButtons(
                QtGui.QMessageBox().No | QtGui.QMessageBox().Yes)
            msgBox.setInformativeText("Remove row {%i} ?" % (rowIndex + 1))
            msgBox.setIcon(QtGui.QMessageBox.Question)
            ret = msgBox.exec_()
        if ret == QtGui.QMessageBox().Yes:
            value = self.getValue()
            self.setValue(value[:rowIndex] + value[rowIndex + 1:], undo=True)

    def _addColumn(self):
        value = self.getValue()  # stored table
        # default value for every new item in the table
        defaultCellsValue = self._delegate.getDefaultValue()
        rows = len(value)

        if (rows > 0):

            columns = len(value[0])
            columnIndex = self._getSelectedIndex()[1]
            if columnIndex is None:
                columnIndex = columns
            for row in value:
                row.insert(columnIndex, defaultCellsValue)
            self.setValue(value, undo=True)
        else:
            self._addRow()

    def _removeColumn(self):
        value = self.getValue()  # stored table
        rows = len(value)
        columnIndex = self._getSelectedIndex()[1]

        if columnIndex is not None:
            msgBox = QtGui.QMessageBox()
            msgBox.setText("Confirmation                ")
            msgBox.setStandardButtons(
                QtGui.QMessageBox().No | QtGui.QMessageBox().Yes)
            msgBox.setInformativeText(
                "Remove column {%i} ?" % (columnIndex + 1))
            msgBox.setIcon(QtGui.QMessageBox.Question)
            ret = msgBox.exec_()

        if (rows > 0) and (columnIndex is not None) and (ret == QtGui.QMessageBox().Yes):
            columns = len(value[0])
            if columns == 1:
                self.setValue([], undo=False)
            else:
                for i in range(len(value)):
                    value[i] = value[i][:columnIndex] + \
                        value[i][columnIndex + 1:]

                self.setValue(value, undo=True)

    def _moveUp(self):
        value = self.getValue()  # stored table
        rows = len(value)
        rowIndex = self._getSelectedIndex()[0]

        if (rows > 1) and (rowIndex is not None) and (rowIndex > 0):
            x = value.pop(rowIndex)
            value.insert(rowIndex - 1, x)
            self.setValue(value, undo=True)
            self._tableView.setCurrentIndex(
                self._tableView.model().index(rowIndex - 1, 0))

    def _moveDown(self):
        value = self.getValue()  # stored table
        rows = len(value)
        rowIndex = self._getSelectedIndex()[0]

        if (rows > 1) and (rowIndex is not None) and (rowIndex < rows - 1):
            x = value.pop(rowIndex)
            value.insert(rowIndex + 1, x)
            self.setValue(value, undo=True)
            self._tableView.setCurrentIndex(
                self._tableView.model().index(rowIndex + 1, 0))

    def setValue(self, value, undo=False):
        if not self._actualValue == value:
            if undo:
                self.emit(QtCore.SIGNAL("valueChanged"), copy.deepcopy(
                    self._actualValue), copy.deepcopy(value))
            self._actualValue = copy.deepcopy(value)

        rows = len(value)
        if (self._format == "2D") and (rows > 0):
            columns = len(value[0])
        else:
            columns = 1

        self._model = QtGui.QStandardItemModel(rows, columns)
        for row in range(rows):
            for column in range(columns):

                index = self._model.index(row, column, QtCore.QModelIndex())
                if (self._format == "1D"):
                    self._model.setData(index, QtCore.QVariant(value[row]))
                if (self._format == "2D"):
                    self._model.setData(
                        index, QtCore.QVariant(value[row][column]))
        self._tableView.setModel(self._model)

    def getValue(self):
        rows = self._model.rowCount()
        columns = self._model.columnCount()
        result = []
        for row in range(rows):
            records = []
            for column in range(columns):
                index = self._model.index(row, column)
                if self._type == "integer":
                    val, bool = self._model.data(index).toInt()
                    records.append(val)
                elif self._type == "float":
                    val, bool = self._model.data(index).toDouble()
                    records.append(val)
                elif self._type == "boolean":
                    records.append(self._model.data(index).toBool())
                else:
                    records.append(str(self._model.data(index).toString()))

            if (self._format == "1D"):
                result.append(records[0])
            if (self._format == "2D"):
                result.append(records)

        return copy.deepcopy(result)

    def _valueChanged(self, value, row, column):
        if (self._format == "1D"):
            if not (value == self._actualValue[row]):
                self._actualValue[row] = value
                self.emit(QtCore.SIGNAL("valueChanged"),
                          self.getValue()[:], self._actualValue[:])

        if (self._format == "2D"):
            if not (value == self._actualValue[row][column]):
                self._actualValue[row][column] = value
                self.emit(QtCore.SIGNAL("valueChanged"),
                          self.getValue()[:], self._actualValue[:])

    @classmethod
    def getDefaultValue(self):
        return []

########### DELGATES ##################


class TableDelegate(QtGui.QItemDelegate):

    def __init__(self, parent=None):
        QtGui.QItemDelegate.__init__(self, parent)

    @abstractmethod
    def createEditor(self, parent, option, index):
        return None

    @abstractmethod
    def setEditorData(self, editor, index):
        pass

    @abstractmethod
    def getDefaultValue(self):
        return None

    def setModelData(self, editor, model, index):
        self.emit(QtCore.SIGNAL("editorValueChanged"),
                  editor.getValue(), index.row(), index.column())
        value = editor.getValue()
        model.setData(index, QtCore.QVariant(value))

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)


class BooleanTableDelegate(TableDelegate):

    def createEditor(self, parent, option, index):
        editor = BooleanComboWidget(parent)
        Qt.QObject.connect(editor, Qt.SIGNAL(
            "currentIndexChanged(int)"), self.updateValue)
        editor.installEventFilter(self)
        return editor

    def updateValue(self, int):
        self.emit(QtCore.SIGNAL("closeEditor()"))

    def setEditorData(self, editor, index):
        value = index.model().data(index, QtCore.Qt.DisplayRole)
        editor.setValue(value.toBool(), undo=True)

    def getDefaultValue(self):
        return BooleanComboWidget.getDefaultValue()


class IntegerTableDelegate(TableDelegate):

    def createEditor(self, parent, option, index):
        editor = IntegerWidget(parent)
        editor.installEventFilter(self)
        return editor

    def setEditorData(self, editor, index):
        value, ok = index.model().data(index, QtCore.Qt.DisplayRole).toInt()
        editor.setValue(value, undo=True)

    def getDefaultValue(self):
        return IntegerWidget.getDefaultValue()


class FloatTableDelegate(TableDelegate):

    def createEditor(self, parent, option, index):
        editor = FloatWidget(parent)
        editor.installEventFilter(self)
        return editor

    def setEditorData(self, editor, index):
        value, ok = index.model().data(index, QtCore.Qt.DisplayRole).toDouble()
        editor.setValue(value, undo=True)

    def getDefaultValue(self):
        return FloatWidget.getDefaultValue()


class StringTableDelegate(TableDelegate):

    def createEditor(self, parent, option, index):
        editor = StringWidget(parent)
        editor.installEventFilter(self)
        return editor

    def setEditorData(self, editor, index):
        value = index.model().data(index, QtCore.Qt.DisplayRole)
        editor.setValue(value.toString(), undo=True)

    def getDefaultValue(self):
        return StringWidget.getDefaultValue()


################## MAIN CLASS ####################
class Save (object):

    def __init__(self, object=None, prevValue=None, nextValue=None):
        self._object = object
        self._prevValue = prevValue
        self._nextValue = nextValue

    def getObject(self):
        return self._object

    def getPrevValue(self):
        return self._prevValue

    def getNextValue(self):
        return self._nextValue


class SettingsWidget(QtGui.QWidget):

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self._formLayout = QtGui.QGridLayout(self)
        self.setUndoHistoryStack(20)
        self._inputWidgts = []
        self._edited = False
        self._properties = None

    def clearLayout(self, layout):
        while layout.count() > 0:
            item = layout.takeAt(0)
            if not item:
                continue
            w = item.widget()
            if w:
                w.deleteLater()

    def _valueChanged(self, value1, value2):
        self._edited = True
        self._undoHistoryStack.append(
            Save(self.sender(), copy.deepcopy(value1), copy.deepcopy(value2)))
        if len(self._undoHistoryStack) > self._historySize:
            self._undoHistoryStack.pop(0)
        item, position = self._historyStackPointer
        if len(self._undoHistoryStack) - 1 >= 0:
            for i in range(item + 1, len(self._undoHistoryStack) - 1):
                self._undoHistoryStack.pop(item + 1)
        self._historyStackPointer = (len(self._undoHistoryStack) - 1, 1)

        self.emit(Qt.SIGNAL("propertyValueChanged()"))

    def setUndoHistoryStack(self, size):
        self._undoHistoryStack = []
        self._historySize = size
        self._historyStackPointer = (-1, 1)  # item, position

    def undo(self):
        item, position = self._historyStackPointer
        if position == 1:
            item = item
            position = 0
        else:
            if item > 0:
                item = item - 1
                position = 0

        self._historyStackPointer = (item, position)
        if (item >= 0) and(item < len(self._undoHistoryStack)):
            self._undoHistoryStack[item].getObject().setValue(
                self._undoHistoryStack[item].getPrevValue(), undo=False)

    def redo(self):
        item, position = self._historyStackPointer
        if position == 0:
            item = item
            position = 1
        else:
            if item < len(self._undoHistoryStack) - 1:
                item = item + 1
                position = 1

        self._historyStackPointer = (item, position)

        if (item >= 0) and(item < len(self._undoHistoryStack)):
            self._undoHistoryStack[item].getObject().setValue(
                self._undoHistoryStack[item].getNextValue(), undo=False)

#    def keyPressEvent(self, event):
#        QtGui.QWidget.keyPressEvent(self, event)
#        undo = (event.modifiers() == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Z)
#        redo = (event.modifiers() == QtCore.Qt.ControlModifier and event.key() == QtCore.Qt.Key_Y)
#        if (undo):
#            self.undo()
#        if (redo):
#            self.redo()

    def setEdited(self, value):
        self._edited = value

    def isEdited(self):
        return self._edited

    def deselect(self):
        for widget in self._inputWidgts:
            if widget != self.sender():
                widget.deselectWidget()

    def getProperties(self):
        values = []
        name = "noName"
        properties = self._properties
        if len(self._inputWidgts) >= 1:
            for inputWidget in self._inputWidgts:
                values.append(inputWidget.getValue())
            name = values.pop(0)

        return (properties, name, values)

    def _nameChanged(self):
        self.emit(Qt.SIGNAL("propertyValueChanged()"))

    def setProperties(self, properties):

        # clean memory
        for item in self._inputWidgts:
            QtCore.QObject.disconnect(item, QtCore.SIGNAL(
                "valueChanged"), self._valueChanged)
        self._inputWidgts = []
        self.clearLayout(self._formLayout)
        self.setUndoHistoryStack(self._historySize)
        self._edited = False

        # create new set of properties

        self._properties = properties

        if properties is not None:
            self._inputWidgts = []
            i = 0
            for property in properties:
                self._nameLabel = QtGui.QLabel(property.get_name())
                #self._formLayout.setWidget(i, QtGui.QFormLayout.LabelRole, self._nameLabel)
                self._formLayout.addWidget(self._nameLabel, i, 0)

                if property.get_format() == "0D":
                    if property.get_type() == "boolean":
                        self._inputWidgts.append(BooleanWidget())
                    elif property.get_type() == "integer":
                        self._inputWidgts.append(IntegerWidget())
                    elif property.get_type() == "float":
                        self._inputWidgts.append(FloatWidget())
                    else:
                        self._inputWidgts.append(StringWidget())

                    spacerItem = QtGui.QSpacerItem(
                        40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
                    self._formLayout.addWidget(self._inputWidgts[i], i, 1)
                    self._formLayout.addItem(spacerItem, i, 2)

                elif (property.get_format() == "1D") or (property.get_format() == "2D"):
                    self._inputWidgts.append(TableWidget(
                        property.get_default_value(), property.get_format(), property.get_type()))
                    self._formLayout.addWidget(
                        self._inputWidgts[i], i, 1, 1, 10)
                    Qt.QObject.connect(self._inputWidgts[i], Qt.SIGNAL(
                        "selected"), self.deselect)

                self._inputWidgts[i].setValue(property.get_default_value())
                QtCore.QObject.connect(self._inputWidgts[i], QtCore.SIGNAL(
                    "valueChanged"), self._valueChanged)
                i = i + 1

        QtCore.QObject.connect(self._inputWidgts[0], QtCore.SIGNAL(
            "textChanged(QString)"), self._nameChanged)


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    properties = []
    properties.append(PropertyInfo("name", "string", "0D", "deviceName"))
    properties.append(PropertyInfo("number0", "integer", "0D", 5))
    properties.append(PropertyInfo("boollll0", "boolean", "0D", False))
    properties.append(PropertyInfo("number1", "float", "0D", 3.5))
    properties.append(PropertyInfo("string2", "string", "0D", "hehe"))
    properties.append(PropertyInfo("table0", "integer", "1D", [1, 2, 3]))
    properties.append(PropertyInfo("tablestringD1", "float", "1D", [1.5]))
    properties.append(PropertyInfo("tablestringD1", "string",
                                   "1D", ["aaaa", "bbb", "ccc"]))
    properties.append(PropertyInfo(
        "tablefloatD1", "float", "1D", [1.0, 2.5, 3.6]))
    properties.append(PropertyInfo("tablebooleanD1",
                                   "boolean", "1D", [True, False, True]))
    properties.append(PropertyInfo(
        "tableboointD1", "integer", "1D", [1, 2, 3]))
    properties.append(PropertyInfo("tablestringD2", "string", "2D", [
                      ["aaaa", "bbb", "ccc"], ["aaaa2", "bbb2", "ccc2"], ["aaaa3", "bbb3", "ccc3"]]))

    nds = SettingsWidget()
    nds.setProperties(properties)
    nds.show()
    sys.exit(app.exec_())



##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains macros that demonstrate the usage of macro parameters"""

from sardana.macroserver.macro import Macro, Type

__all__ = ["pt0", "pt1", "pt2", "pt3", "pt3d", "pt4", "pt5", "pt6", "pt7",
           "pt7d1", "pt7d2", "pt8", "pt9", "pt10", "pt11", "pt12", "pt13",
           "pt14", "pt14d", "twice"]


class pt0(Macro):
    """Macro without parameters. Pretty dull.
       Usage from Spock, ex.:
       pt0
       """

    param_def = []

    def run(self):
        pass


class pt1(Macro):
    """Macro with one float parameter: Each parameter is described in the
    param_def sequence as being a sequence of four elements: name, type,
    default value and description.
    Usage from Spock, ex.:
    pt1 1
    """

    param_def = [['value', Type.Float, None, 'some bloody float']]

    def run(self, f):
        pass


class pt1d(Macro):
    """Macro with one float parameter with default value..
    Usage from Spock, ex.:
    pt1d 1
    pt1d
    """

    param_def = [['value', Type.Float, None, 'some bloody float']]

    def run(self, f):
        pass


class pt2(Macro):
    """Macro with one Motor parameter: Each parameter is described in the
    param_def sequence as being a sequence of four elements: name, type,
    default value and description.
    Usage from Spock, ex.
    pt2 mot1
    """

    param_def = [['motor', Type.Motor, None, 'some bloody motor']]

    def run(self, m):
        pass


class pt3(Macro):
    """Macro with a list of numbers as parameter: the type is a sequence of
    parameter types which is repeated. In this case it is a repetition of a
    float so only one parameter is defined.
    By default the repetition as a semantics of 'at least one'
    Usages from Spock, ex.:
    pt3 [1 34 15]
    pt3 1 34 15
    """

    param_def = [
        ['numb_list', [['pos', Type.Float, None, 'value']], None, 'List of values'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt3d(Macro):
    """Macro with a list of numbers as parameter: the type is a sequence of
    parameter types which is repeated. In this case it is a repetition of a
    float so only one parameter is defined. The parameter has a default value.
    By default the repetition as a semantics of 'at least one'
    Usages from Spock, ex.:
    pt3d [1 34 15]
    pt3d 1 34 15
    Usage taken the default value, ex.:
    pt3d [1 [] 15]
    """

    param_def = [
        ['numb_list', [['pos', Type.Float, 21, 'value']], None, 'List of values'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt4(Macro):
    """Macro with a list of motors as parameter: the type is a sequence of
    parameter types which is repeated. In this case it is a repetition of a
    motor so only one parameter is defined.
    By default the repetition as a semantics of 'at least one'.
    Usages from Spock, ex.:
    pt4 [mot1 mot2 mot3]
    pt4 mot1 mot2 mot3
    """

    param_def = [
        ['motor_list', [['motor', Type.Motor, None, 'motor name']],
            None, 'List of motors'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt5(Macro):
    """Macro with a motor parameter followed by a list of numbers.
    Usages from Spock, ex.:
    pt5 mot1 [1 3]
    pt5 mot1 1 3
    """

    param_def = [
        ['motor', Type.Motor, None, 'Motor to move'],
        ['numb_list', [['pos', Type.Float, None, 'value']], None, 'List of values'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt6(Macro):
    """Macro with a motor parameter followed by a list of numbers. The list as
    explicitly stated an optional last element which is a dictionary that defines the
    min and max values for repetitions.
    Usages from Spock, ex.:
    pt6 mot1 [1 34 1]
    pt6 mot1 1 34 1
    """

    param_def = [
        ['motor', Type.Motor, None, 'Motor to move'],
        ['numb_list', [['pos', Type.Float, None, 'value'], {
            'min': 1, 'max': None}], None, 'List of values'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt7(Macro):
    """Macro with a list of pair Motor,Float.
    Usages from Spock, ex.:
    pt7 [[mot1 1] [mot2 3]]
    pt7 mot1 1 mot2 3
    """

    param_def = [
        ['m_p_pair', [['motor', Type.Motor, None, 'Motor to move'],
                      ['pos',   Type.Float, None, 'Position to move to']],
         None, 'List of motor/position pairs']
    ]

    def run(self, *args, **kwargs):
        pass


class pt7d1(Macro):
    """Macro with a list of pair Motor,Float. Default value for last
    repeat parameter element.
    Usages from Spock, ex.:
    pt7d1 [[mot1 1] [mot2 3]]
    pt7d1 mot1 1 mot2 3
    Using default value, ex.:
    pt7d1 [[mot1] [mot2 3]] # at any repetition
    pt7d1 mot1 # if only one repetition

    """

    param_def = [
        ['m_p_pair', [['motor', Type.Motor, None, 'Motor to move'],
                      ['pos',   Type.Float, 2, 'Position to move to']],
         None, 'List of motor/position pairs']
    ]

    def run(self, *args, **kwargs):
        pass


class pt7d2(Macro):
    """Macro with a list of pair Motor,Float. Default value for both
    repeat parameters elements.
    Usages from Spock, ex.:
    pt7d2 [[mot1 1] [mot2 3]]
    pt7d2 mot1 1 mot2 3
    Using both default values, ex.:
    pt7d2 [[] [mot2 3] []] # at any repetition
    """

    param_def = [
        ['m_p_pair', [['motor', Type.Motor, 'mot1', 'Motor to move'],
                      ['pos',   Type.Float, 2, 'Position to move to']],
         None, 'List of motor/position pairs']
    ]

    def run(self, *args, **kwargs):
        pass


class pt8(Macro):
    """Macro with a list of pair Motor,Float. The min and max elements have been
    explicitly stated.
    Usages from Spock, ex.:
    pt8 [[mot1 1] [mot2 3]]
    pt8 mot1 1 mot2 3
    """

    param_def = [
        ['m_p_pair', [['motor', Type.Motor, None, 'Motor to move'],
                      ['pos',   Type.Float, None, 'Position to move to'],
                      {'min': 1, 'max': 2}],
         None, 'List of motor/position pairs']
    ]

    def run(self, *args, **kwargs):
        pass


class pt9(Macro):
    """Same as macro pt7 but with min and max number of repetitions of the
    repeat parameter.
    Usages from Spock, ex.:
    pt9 [[mot1 1][mot2 3]]
    pt9 mot1 1 mot2 3
    """

    param_def = [
        ['m_p_pair', [['motor', Type.Motor, None, 'Motor to move'],
                      ['pos',  Type.Float, None, 'Position to move to'],
                      {'min': 1, 'max': 2}],
         None, 'List of motor/position pairs'],
    ]

    def run(self, *args, **kwargs):
        pass


class pt10(Macro):
    """Macro with list of numbers followed by a motor parameter. The repeat
    parameter may be defined as first one.
    Usage from Spock, ex.:
    pt10 [1 3] mot1
    pt10 1 mot1 # if only one repetition
    """

    param_def = [
        ['numb_list', [['pos', Type.Float, None, 'value']], None, 'List of values'],
        ['motor', Type.Motor, None, 'Motor to move']
    ]

    def run(self, *args, **kwargs):
        pass


class pt11(Macro):
    """Macro with counter parameter followed by a list of numbers, followed by
    a motor parameter. The repeat parameter may be defined in between other
    parameters.
    Usages from Spock, ex.:
    pt11 ct1 [1 3] mot1
    pt11 ct1 1 mot1 # if only one repetition
    """

    param_def = [
        ['counter', Type.ExpChannel, None, 'Counter to count'],
        ['numb_list', [['pos', Type.Float, None, 'value']], None, 'List of values'],
        ['motor', Type.Motor, None, 'Motor to move']
    ]

    def run(self, *args, **kwargs):
        pass


class pt12(Macro):
    """Macro with list of motors followed by list of numbers. Two repeat
    parameters may defined.
    Usage from Spock, ex.:
    pt12 [1 3 4] [mot1 mot2]
    pt12 1 mot1 # if only one repetition for each repeat parameter
    """

    param_def = [
        ['numb_list', [['pos', Type.Float, None, 'value']], None, 'List of values'],
        ['motor_list', [['motor', Type.Motor, None, 'Motor to move']],
         None, 'List of motors']
    ]

    def run(self, *args, **kwargs):
        pass


class pt13(Macro):
    """Macro with list of motors groups, where each motor group is a list of
    motors. Repeat parameters may be defined as nested.
    Usage from Spock, ex.:
    pt13 [[mot1 mot2] [mot3 mot4]]
"""

    param_def = [
        ['motor_group_list',
         [['motor_list', [['motor', Type.Motor, None, 'Motor to move']],
           None, 'List of motors']],
         None, 'Motor groups']
    ]

    def run(self, *args, **kwargs):
        pass


class pt14(Macro):
    """Macro with list of motors groups, where each motor group is a list of
    motors and a float. Repeat parameters may be defined as nested.
    Usage from Spock, ex.:
    pt14 [[[mot1 mot2] 3] [[mot3] 5]]
    """

    param_def = [
        ['motor_group_list',
         [['motor_list', [['motor', Type.Motor, None, 'Motor to move']], None, 'List of motors'],
          ['float', Type.Float, None, 'Number']],
         None, 'Motor groups']
    ]

    def run(self, *args, **kwargs):
        pass


class pt14d(Macro):
    """Macro with list of motors groups, where each motor group is a list of
    motors and a float. Repeat parameters may be defined as nested.
    Default values can be used.
    Usages taken default values, ex.:
    pt14d [[[mot1 mot2] 3] [[mot3] []]]
    pt14d [[[mot1 []] 3] [[mot3] []]]
    pt14d [[[[]] 3] [[mot3] []]]
    """

    param_def = [
        ['motor_group_list',
         [['motor_list', [['motor', Type.Motor, 'mot1', 'Motor to move']], None, 'List of motors'],
          ['float', Type.Float, 33, 'Number']],
         None, 'Motor groups']
    ]

    def run(self, *args, **kwargs):
        pass


class twice(Macro):
    """A macro that returns a float that is twice its input. It also sets its
    data to be a dictionary with 'in','out' as keys and value,result
    as values, respectively"""

    # uncomment the following lines as necessary. Otherwise you may delete them
    param_def = [["value", Type.Float, 23, "value to be doubled"]]
    result_def = [["result", Type.Float, None,
                   "the double of the given value"]]
    #hints = {}
    # env = (,)

    # uncomment the following lines if need prepare. Otherwise you may delete them
    # def prepare(self):
    #    pass

    def run(self, n):
        ret = 2 * n
        self.setData({'in': n, 'out': ret})
        return ret
